# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from dataclasses import dataclass
from typing import List, Optional

from pontos.github.models.base import App, GitHubModel, Team, User

__all__ = (
    "BranchProtection",
    "BranchProtectionFeature",
    "BypassPullRequestAllowances",
    "DismissalRestrictions",
    "Restrictions",
    "RequiredStatusChecks",
    "StatusCheck",
    "RequiredPullRequestReviews",
    "RequiredStatusChecks",
    "BypassPullRequestAllowances",
    "DismissalRestrictions",
)


@dataclass
class DismissalRestrictions(GitHubModel):
    """
    Settings to only allow specific users, teams and apps to dismiss pull
    request reviews

    Attributes:
        url: URL to the dismissal restrictions
        users_url: URL to the users of the dismissal restrictions
        teams_url: URL to the teams of the dismissal restrictions
        users: List of user allowed to dismiss pull request reviews
        teams: List of teams allowed to dismiss pull request reviews
        apps: List of apps allowed to dismiss pull request reviews
    """

    url: str
    users_url: str
    teams_url: str
    users: List[User]
    teams: List[Team]
    apps: List[App]


@dataclass
class BypassPullRequestAllowances(GitHubModel):
    """
    Settings to allow users, teams and apps to bypass pull request reviews

    Attributes:
        user: List of user allowed to bypass required pull request reviews
        teams: List of teams allowed to bypass required pull request reviews
        apps: List of apps allowed to bypass required pull request reviews
    """

    users: List[User]
    teams: List[Team]
    apps: List[App]


@dataclass
class RequiredPullRequestReviews(GitHubModel):
    """
    Requires pull request review settings of a branch protection

    Attributes:
        url: URL to the required pull request reviews
        dismiss_stale_reviews: Dismiss stale reviews
        require_code_owner_reviews: Require reviews by code owners
        required_approving_review_count: Number of approvals required
        require_last_push_approval: Require to approve the last push
        dismissal_restrictions: Restrictions for who can dismiss pull request
            reviews
        bypass_pull_request_allowances: Settings for allowing bypassing the
            required pull request reviews
    """

    url: str
    dismiss_stale_reviews: bool
    require_code_owner_reviews: bool
    required_approving_review_count: int
    require_last_push_approval: bool
    dismissal_restrictions: Optional[DismissalRestrictions] = None
    bypass_pull_request_allowances: Optional[BypassPullRequestAllowances] = None


@dataclass
class StatusCheck(GitHubModel):
    """
    Status check

    Attributes:
        context:
        app: App ID as the source of the status check
    """

    context: str
    app_id: Optional[int] = None


@dataclass
class RequiredStatusChecks(GitHubModel):
    """
    Required status checks settings of a branch protection

    Attributes:
        url: URL to the required status checks
        strict: True to require status checks to pass before merging
        checks: List of status checks
        enforcement_level: Enforcement level of the required status checks
    """

    url: str
    strict: bool
    checks: List[StatusCheck]
    enforcement_level: Optional[str] = None


@dataclass
class Restrictions(GitHubModel):
    """
    Branch protection push restrictions

    Attributes:
        url: URL to the restrictions
        users_url: URL to the users of the restrictions
        teams_url: URL to the teams of the restrictions
        apps_url: URL to the apps of the restrictions
        users: List of restricted users
        teams: List of restricted teams
        apps: List of restricted apps
    """

    url: str
    users_url: str
    teams_url: str
    apps_url: str
    users: List[User]
    teams: List[Team]
    apps: List[App]


@dataclass
class BranchProtectionFeature(GitHubModel):
    """
    GitHub branch protection feature setting

    Attributes:
        enable: True if the feature is enabled
        url: REST API URL to change the feature
    """

    enabled: bool
    url: Optional[str] = None


@dataclass
class BranchProtection(GitHubModel):
    """
    GitHub branch protection information

    Attributes:
        url: URL to the branch protection rules
        required_status_checks: Required status check for the matching branches
        required_pull_request_reviews: Required pull request reviews for the
            matching branches.
        restrictions: Restrictions who can push to the matching branches.
        enforce_admins: Enforce the rules also for user in a admin role.
        required_linear_history: Require a linear history before merging.
            Restricts merging if the matching branch is out of date.
        allow_force_pushes: Allow force pushes to the matching branches.
        allow_deletions: Allow to delete the matching branches.
        block_creations: Restrict pushes that create matching branches.
        required_conversation_resolution: Require conversation resolution before
            merging.
        lock_branch: Mark matching branches as read-only. Users cannot push to
            matching branches.
        allow_fork_syncing: Whether users can pull changes from upstream when
            the matching branch is locked.
        required_signatures: Require git commit signatures.
    """

    url: str
    required_status_checks: Optional[RequiredStatusChecks] = None
    required_pull_request_reviews: Optional[RequiredPullRequestReviews] = None
    restrictions: Optional[Restrictions] = None
    enforce_admins: Optional[BranchProtectionFeature] = None
    required_linear_history: Optional[BranchProtectionFeature] = None
    allow_force_pushes: Optional[BranchProtectionFeature] = None
    allow_deletions: Optional[BranchProtectionFeature] = None
    block_creations: Optional[BranchProtectionFeature] = None
    required_conversation_resolution: Optional[BranchProtectionFeature] = None
    lock_branch: Optional[BranchProtectionFeature] = None
    allow_fork_syncing: Optional[BranchProtectionFeature] = None
    required_signatures: Optional[BranchProtectionFeature] = None



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import asyncio
from argparse import Namespace
from typing import Callable

import httpx

from pontos.nvd.cpe.api import CPEApi

from ._parser import cpe_parser, cpes_parser

__all__ = ("CPEApi",)


async def query_cpe(args: Namespace) -> None:
    async with CPEApi(token=args.token) as api:
        cpe = await api.cpe(args.cpe_name_id)
        print(cpe)


async def query_cpes(args: Namespace) -> None:
    async with CPEApi(token=args.token) as api:
        response = api.cpes(
            keywords=args.keywords,
            cpe_match_string=args.cpe_match_string,
            request_results=args.number,
            start_index=args.start,
        )
        async for cpe in response:
            print(cpe)


def cpe_main() -> None:
    main(cpe_parser(), query_cpe)


def cpes_main() -> None:
    main(cpes_parser(), query_cpes)


def main(args: Namespace, func: Callable) -> None:
    try:
        asyncio.run(func(args))
    except KeyboardInterrupt:
        pass
    except httpx.HTTPStatusError as e:
        print(f"HTTP Error {e.response.status_code}: {e.response.text}")



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from datetime import datetime
from types import TracebackType
from typing import (
    Iterable,
    Iterator,
    List,
    Optional,
    Type,
    Union,
)

from httpx import Timeout

from pontos.errors import PontosError
from pontos.nvd.api import (
    DEFAULT_TIMEOUT_CONFIG,
    JSON,
    NVDApi,
    NVDResults,
    Params,
    convert_camel_case,
    format_date,
    now,
)
from pontos.nvd.models.cve import CVE
from pontos.nvd.models.cvss_v2 import Severity as CVSSv2Severity
from pontos.nvd.models.cvss_v3 import Severity as CVSSv3Severity

__all__ = ("CVEApi",)

DEFAULT_NIST_NVD_CVES_URL = "https://services.nvd.nist.gov/rest/json/cves/2.0"
MAX_CVES_PER_PAGE = 2000


def _result_iterator(data: JSON) -> Iterator[CVE]:
    vulnerabilities: Iterable = data.get("vulnerabilities", [])  # type: ignore
    return (
        CVE.from_dict(vulnerability["cve"]) for vulnerability in vulnerabilities
    )


class CVEApi(NVDApi):
    """
    API for querying the NIST NVD CVE information.

    Should be used as an async context manager.

    Example:
        .. code-block:: python

            from pontos.nvd.cve import CVEApi

            async with CVEApi() as api:
                cve = await api.cve("CVE-2022-45536")
    """

    def __init__(
        self,
        *,
        token: Optional[str] = None,
        timeout: Optional[Timeout] = DEFAULT_TIMEOUT_CONFIG,
        rate_limit: bool = True,
        request_attempts: int = 1,
    ) -> None:
        """
        Create a new instance of the CVE API.

        Args:
            token: The API key to use. Using an API key allows to run more
                requests at the same time.
            timeout: Timeout settings for the HTTP requests
            rate_limit: Set to False to ignore rate limits. The public rate
                limit (without an API key) is 5 requests in a rolling 30 second
                window. The rate limit with an API key is 50 requests in a
                rolling 30 second window.
                See https://nvd.nist.gov/developers/start-here#divRateLimits
                Default: True.
            request_attempts: The number of attempts per HTTP request. Defaults to 1.
        """
        super().__init__(
            DEFAULT_NIST_NVD_CVES_URL,
            token=token,
            timeout=timeout,
            rate_limit=rate_limit,
            request_attempts=request_attempts,
        )

    def cves(
        self,
        *,
        last_modified_start_date: Optional[datetime] = None,
        last_modified_end_date: Optional[datetime] = None,
        published_start_date: Optional[datetime] = None,
        published_end_date: Optional[datetime] = None,
        cpe_name: Optional[str] = None,
        is_vulnerable: Optional[bool] = None,
        cvss_v2_vector: Optional[str] = None,
        cvss_v2_severity: Optional[CVSSv2Severity] = None,
        cvss_v3_vector: Optional[str] = None,
        cvss_v3_severity: Optional[CVSSv3Severity] = None,
        keywords: Optional[Union[List[str], str]] = None,
        cwe_id: Optional[str] = None,
        source_identifier: Optional[str] = None,
        virtual_match_string: Optional[str] = None,
        has_cert_alerts: Optional[bool] = None,
        has_cert_notes: Optional[bool] = None,
        has_kev: Optional[bool] = None,
        has_oval: Optional[bool] = None,
        request_results: Optional[int] = None,
        start_index: int = 0,
        results_per_page: Optional[int] = None,
    ) -> NVDResults[CVE]:
        """
        Get all CVEs for the provided arguments

        https://nvd.nist.gov/developers/vulnerabilities#divGetCves

        Args:
            last_modified_start_date: Return all CVEs modified after this date.
            last_modified_end_date: Return all CVEs modified before this date.
                If last_modified_start_date is set but no
                last_modified_end_date is passed it is set to now.
            published_start_date: Return all CVEs that were added to the NVD
                (i.e., published) after this date.
            published_end_date: Return all CVEs that were added to the NVD
                (i.e., published) before this date. If published_start_date is
                set but no published_end_date is passed it is set to now.
            cpe_name: Return all CVEs associated with a specific CPE. The exact
                value provided with cpe_name is compared against the CPE Match
                Criteria within a CVE applicability statement. If the value of
                cpe_name is considered to match, the CVE is included in the
                results.
            is_vulnerable: Return only CVEs that match cpe_name that are
                vulnerable. Requires cpe_name to be set.
            cvss_v2_vector: Return all CVEs matching this CVSSv2 vector
            cvss_v2_severity: Return all CVEs matching the CVSSv2 severity
            cvss_v3_vector: Return all CVEs matching this CVSSv3 vector
            cvss_v3_severity: Return all CVEs matching the CVSSv3 severity
            keywords: Returns only the CVEs where a word or phrase is found in
                the current description.
            cwe_id: Returns only the CVEs that include a weakness identified by
                Common Weakness Enumeration using the provided cwe_id.
            source_identifier: Returns CVEs where the exact value of
                source_identifier appears as a data source in the CVE record.
                For example: cve@mitre.org
            virtual_match_string: Filters CVEs more broadly than cpe_name. The
                exact value of virtual_match_string is compared against the CPE
                Match Criteria present on CVE applicability statements. If
                cpe_name and virtual_match_string are provided only cpe_name is
                considered.
            has_cert_alerts: Returns the CVEs that contain a Technical Alert
                from US-CERT.
            has_cert_notes: Returns the CVEs that contain a Vulnerability Note
                from CERT/CC.
            has_kev: Returns the CVE that appear in CISA's Known Exploited
                Vulnerabilities (KEV) Catalog.
            has_oval: Returns the CVEs that contain information from MITRE's
                Open Vulnerability and Assessment Language (OVAL) before this
                transitioned to the Center for Internet Security (CIS).
            request_results: Number of CVEs to download. Set to None (default)
                to download all available CVEs.
            start_index: Index of the first CVE to be returned. Useful only for
                paginated requests that should not start at the first page.
            results_per_page: Number of results in a single requests. Mostly
                useful for paginated requests.

        Returns:
            A NVDResponse for CVEs

        Examples:
            .. code-block:: python

                from pontos.nvd.cve import CVEApi

                async with CVEApi() as api:
                    async for cve in api.cves(keywords=["Mac OS X", "kernel"]):
                        print(cve.id)

                    json = await api.cves(
                        cpe_name="cpe:2.3:o:microsoft:windows_7:-:*:*:*:*:*:x64:*",
                    ).json()

                    async for cves in api.cves(
                        virtual_match_string="cpe:2.3:o:microsoft:windows_7:-:*:*:*:*:*:x64:*",
                    ).chunks():
                        for cve in cves:
                            print(cve)
        """
        params: Params = {}
        if last_modified_start_date:
            params["lastModStartDate"] = format_date(last_modified_start_date)
            if not last_modified_end_date:
                params["lastModEndDate"] = format_date(now())
        if last_modified_end_date:
            params["lastModEndDate"] = format_date(last_modified_end_date)

        if published_start_date:
            params["pubStartDate"] = format_date(published_start_date)
            if not published_end_date:
                params["pubEndDate"] = format_date(now())
        if published_end_date:
            params["pubEndDate"] = format_date(published_end_date)

        if cpe_name:
            params["cpeName"] = cpe_name
            if is_vulnerable:
                params["isVulnerable"] = ""

        if cvss_v2_vector:
            params["cvssV2Metrics"] = cvss_v2_vector
        if cvss_v3_vector:
            params["cvssV3Metrics"] = cvss_v3_vector
        if cvss_v2_severity:
            params["cvssV2Severity"] = cvss_v2_severity.value
        if cvss_v3_severity:
            params["cvssV3Severity"] = cvss_v3_severity.value

        if keywords:
            if isinstance(keywords, str):
                keywords = [keywords]

            params["keywordSearch"] = " ".join(keywords)
            if any((" " in keyword for keyword in keywords)):
                params["keywordExactMatch"] = ""

        if cwe_id:
            params["cweId"] = cwe_id

        if source_identifier:
            params["sourceIdentifier"] = source_identifier

        if not cpe_name and virtual_match_string:
            params["virtualMatchString"] = virtual_match_string

        if has_cert_alerts:
            params["hasCertAlerts"] = ""
        if has_cert_notes:
            params["hasCertNotes"] = ""
        if has_kev:
            params["hasKev"] = ""
        if has_oval:
            params["hasOval"] = ""

        results_per_page = min(
            results_per_page or MAX_CVES_PER_PAGE,
            request_results or MAX_CVES_PER_PAGE,
        )
        return NVDResults(
            self,
            params,
            _result_iterator,
            request_results=request_results,
            results_per_page=results_per_page,
            start_index=start_index,
        )

    async def cve(self, cve_id: str) -> CVE:
        """
        Returns a single CVE matching the CVE ID. Vulnerabilities not yet
        published in the NVD are not available.

        Args:
            cve_id: Common Vulnerabilities and Exposures identifier

        Returns:
            A CVE matching the CVE ID

        Raises:
            PontosError: If CVE ID is empty or if no CVE with the CVE ID is
                found.

        Example:
            .. code-block:: python

                from pontos.nvd.cve import CVEApi

                async with CVEApi() as api:
                    cve = await api.cve("CVE-2022-45536")
                    print(cve)
        """
        if not cve_id:
            raise PontosError("Missing CVE ID.")

        response = await self._get(params={"cveId": cve_id})
        response.raise_for_status()
        data = response.json(object_hook=convert_camel_case)
        vulnerabilities = data["vulnerabilities"]
        if not vulnerabilities:
            raise PontosError(f"No CVE with CVE ID '{cve_id}' found.")

        vulnerability = vulnerabilities[0]
        return CVE.from_dict(vulnerability["cve"])

    async def __aenter__(self) -> "CVEApi":
        await super().__aenter__()
        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> Optional[bool]:
        return await super().__aexit__(  # type: ignore
            exc_type, exc_value, traceback
        )



# SPDX-FileCopyrightText: 2021-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import json
from dataclasses import dataclass
from enum import Enum
from pathlib import Path
from typing import Any, Dict, Iterable, Optional


class PullRequestState(Enum):
    """
    State of a pull request

    Attributes:
        OPEN: The pull request is open
        CLOSED: The pull request is closed
    """

    OPEN = "open"
    CLOSED = "closed"


@dataclass
class Label:
    """
    A label of a pull request or issue
    """

    name: str


@dataclass
class Ref:
    """
    A git branch reference

    Attributes:
        name: Name of the git branch reference for example main
        sha: Git commit ID of the reference
    """

    name: str
    sha: str


@dataclass
class GitHubPullRequestEvent:
    """
    Event data of a GitHub Pull Request

    https://docs.github.com/en/developers/webhooks-and-events/webhooks/webhook-events-and-payloads#pull_request

    Attributes:
        draft: True if the pull request is a draft
        number: ID of the pull request
        labels: Labels attached to the pull request
        title: Title of the pull request
        merged: True if the pull request is already merged
        state: State of the pull request (open, closed)
        base: Base reference of the pull request (target branch)
        head: Head reference of the pull request (source branch)
    """

    draft: Optional[bool]
    number: Optional[int]
    labels: Optional[Iterable[str]]
    title: Optional[str]
    merged: Optional[bool]
    state: PullRequestState
    base: Ref
    head: Ref

    def __init__(self, pull_request_data: Dict[str, Any]):
        """
        Derive the pull request information from the pull request data of a
        GitHub event.

        Args:
            pull_request_data: JSON based pull request information as dict
        """
        data = pull_request_data or {}

        self.draft = data.get("draft")
        self.number = data.get("number")
        self.labels = [Label(label.get("name")) for label in data.get("labels")]  # type: ignore #pylint: disable=line-too-long # noqa: E501
        self.title = data.get("title")
        self.merged = data.get("merged")
        self.state = PullRequestState(data.get("state"))

        base = data.get("base") or {}
        self.base = Ref(base.get("ref"), base.get("sha"))  # type: ignore

        head = data.get("head") or {}
        self.head = Ref(head.get("ref"), head.get("sha"))  # type: ignore


@dataclass
class GitHubEvent:
    """
    GitHub Actions provides event data for the running action as JSON data in
    a local file at the runner.

    The JSON data for the events is specified at
    https://docs.github.com/en/developers/webhooks-and-events/webhooks/webhook-events-and-payloads

    Attributes:
        pull_request: Information about the pull request
    """

    pull_request: GitHubPullRequestEvent

    def __init__(self, event_path: Path):
        """
        Loads the event data from the passed path

        Args:
            event_path: Path to the event data
        """
        content = event_path.read_text(encoding="utf-8")
        self._event_data = json.loads(content) if content else {}
        pull_request_data = self._event_data.get("pull_request")
        self.pull_request = (
            GitHubPullRequestEvent(pull_request_data)  # type: ignore
            if pull_request_data
            else None
        )

    def __str__(self) -> str:
        return json.dumps(self._event_data, indent=2)



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


import unittest

from pontos.version import VersionError


class VersionErrorTestCase(unittest.TestCase):
    def test_should_print_message(self):
        err = VersionError("foo bar")
        self.assertEqual(str(err), "foo bar")

    def test_should_raise(self):
        with self.assertRaisesRegex(VersionError, "^foo bar$"):
            raise VersionError("foo bar")



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import os
import unittest
from pathlib import Path

from pontos.git._status import Status, StatusEntry, parse_git_status


class StatusEntryTestCase(unittest.TestCase):
    def test_parse_modified_modified(self):
        status = StatusEntry("MM foo.txt")

        self.assertEqual(status.index, Status.MODIFIED)
        self.assertEqual(status.working_tree, Status.MODIFIED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_parse_modified_unmodified(self):
        status = StatusEntry("M  foo.txt")

        self.assertEqual(status.index, Status.MODIFIED)
        self.assertEqual(status.working_tree, Status.UNMODIFIED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_parse_deleted(self):
        status = StatusEntry("D  foo.txt")

        self.assertEqual(status.index, Status.DELETED)
        self.assertEqual(status.working_tree, Status.UNMODIFIED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_parse_added(self):
        status = StatusEntry("A  foo.txt")

        self.assertEqual(status.index, Status.ADDED)
        self.assertEqual(status.working_tree, Status.UNMODIFIED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_parse_untracked(self):
        status = StatusEntry("?? foo.txt")

        self.assertEqual(status.index, Status.UNTRACKED)
        self.assertEqual(status.working_tree, Status.UNTRACKED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_parse_ignored_untracked(self):
        status = StatusEntry("!? foo.txt")

        self.assertEqual(status.index, Status.IGNORED)
        self.assertEqual(status.working_tree, Status.UNTRACKED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_pathlike(self):
        status = StatusEntry("MM foo.txt")
        self.assertEqual(os.fspath(status), "foo.txt")


class ParseGitStatusTestCase(unittest.TestCase):
    def test_parse_git_status(self):
        output = (
            b" M bar.json\x00R  foo.rst\x00foo.md\x00A  foo.txt\x00"
            b"MM ipsum.json\x00AM ipsum.txt\x00D  lorem.json\x00"
        )

        it = parse_git_status(output.decode("utf-8"))

        status = next(it)
        self.assertEqual(status.index, Status.UNMODIFIED)
        self.assertEqual(status.working_tree, Status.MODIFIED)
        self.assertEqual(status.path, Path("bar.json"))

        status = next(it)
        self.assertEqual(status.index, Status.RENAMED)
        self.assertEqual(status.working_tree, Status.UNMODIFIED)
        self.assertEqual(status.path, Path("foo.rst"))
        self.assertEqual(status.old_path, Path("foo.md"))

        status = next(it)
        self.assertEqual(status.index, Status.ADDED)
        self.assertEqual(status.working_tree, Status.UNMODIFIED)
        self.assertEqual(status.path, Path("foo.txt"))

        status = next(it)
        self.assertEqual(status.index, Status.MODIFIED)
        self.assertEqual(status.working_tree, Status.MODIFIED)
        self.assertEqual(status.path, Path("ipsum.json"))

        status = next(it)
        self.assertEqual(status.index, Status.ADDED)
        self.assertEqual(status.working_tree, Status.MODIFIED)
        self.assertEqual(status.path, Path("ipsum.txt"))

        status = next(it)
        self.assertEqual(status.index, Status.DELETED)
        self.assertEqual(status.working_tree, Status.UNMODIFIED)
        self.assertEqual(status.path, Path("lorem.json"))

        with self.assertRaises(StopIteration):
            next(it)



# SPDX-FileCopyrightText: 2019-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

"""Script to update the year of last modification
in the license header of source code files.\n
Also it appends a header if it is missing in the file.
"""

import io
import re
import sys
from dataclasses import dataclass
from functools import cache
from pathlib import Path
from typing import Optional, Sequence, Union

from pontos.errors import PontosError
from pontos.git import Git
from pontos.terminal.null import NullTerminal
from pontos.terminal.rich import RichTerminal

from ._parser import parse_args

SUPPORTED_FILE_TYPES = [
    ".bash",
    ".c",
    ".h",
    ".go",
    ".cmake",
    ".js",
    ".nasl",
    ".po",
    ".py",
    ".sh",
    ".ts",
    ".tsx",
    ".txt",
    ".xml",
    ".xsl",
]
OLD_LINES = [
    "# \-\*\- coding: utf\-8 \-\*\-",
    "This program is free software: you can redistribute it and/or modify",
    "it under the terms of the GNU Affero General Public License as",
    "published by the Free Software Foundation, either version 3 of the",
    "License, or \(at your option\) any later version.",
    "This program is free software; you can redistribute it and/or",
    "modify it under the terms of the GNU General Public License",
    "version 2 as published by the Free Software Foundation.",
    "This program is free software: you can redistribute it and/or modify",
    "it under the terms of the GNU General Public License as published by",
    "the Free Software Foundation, either version 3 of the License, or",
    "\(at your option\) any later version.",
    "This program is distributed in the hope that it will be useful,",
    "but WITHOUT ANY WARRANTY; without even the implied warranty of",
    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the",
    "GNU Affero General Public License for more details.",
    "GNU General Public License for more details.",
    "You should have received a copy of the GNU Affero General Public License",
    "You should have received a copy of the GNU General Public License",
    "along with this program.  If not, see <http://www.gnu.org/licenses/>.",
    "along with this program; if not, write to the Free Software",
    "Foundation, Inc\., 51 Franklin St, Fifth Floor, Boston, MA 02110\-1301 USA\.",  # noqa: E501
]


def _get_modified_year(f: Path) -> str:
    """In case of the changed arg, update year to last modified year"""
    try:
        ret = Git().log("-1", "--date=format:%Y", str(f), format="%ad")[0]
    except IndexError:
        raise PontosError(f'Empty "git log -1" output for {f}.')

    return ret


@dataclass
class CopyrightMatch:
    creation_year: str
    modification_year: Optional[str]
    company: str


def _find_copyright(
    line: str,
    copyright_regex: re.Pattern,
) -> tuple[bool, Union[CopyrightMatch, None]]:
    """Match the line for the copyright_regex"""
    copyright_match = re.search(copyright_regex, line)
    if copyright_match:
        return (
            True,
            CopyrightMatch(
                creation_year=copyright_match.group(2),
                modification_year=copyright_match.group(3),
                company=copyright_match.group(4),
            ),
        )
    return False, None


def _add_header(
    suffix: str, license_id: str, company: str, year: str
) -> Union[str, None]:
    """Tries to add the header to the file.
    Requirements:
      - file type must be supported
      - license file must exist
    """
    if suffix in SUPPORTED_FILE_TYPES:
        root = Path(__file__).parent
        license_file = root / "templates" / license_id / f"template{suffix}"
        try:
            return (
                license_file.read_text(encoding="utf-8")
                .replace("<company>", company)
                .replace("<year>", year)
            )
        except FileNotFoundError as e:
            raise e
    else:
        raise ValueError


def _remove_outdated_lines(
    content: str, cleanup_regexes: list[re.Pattern]
) -> Optional[str]:
    """Remove lines that contain outdated copyright header ..."""
    changed = False
    splitted_lines = content.splitlines()
    i = 0
    for line in splitted_lines[:20]:
        if i > 3 and re.match(r"^(([#*]|//) ?$)", line):
            splitted_lines.pop(i)
            continue
        for regex in cleanup_regexes:
            if regex.match(line):
                changed = True
                splitted_lines.pop(i)
                i = i - 1
                break
        i = i + 1
    if changed:
        new_content = "\n".join(splitted_lines) + "\n"
        return new_content
    return None


def update_file(
    file: Path,
    year: str,
    license_id: str,
    company: str,
    *,
    cleanup: bool = False,
    single_year: bool = False,
) -> None:
    """Function to update the header of the given file

    Checks if header exists. If not it adds an header to that file, otherwise it
    checks if year is up to date
    """

    copyright_regex = _compile_copyright_regex()
    cleanup_regexes = _compile_outdated_regex() if cleanup else None

    try:
        with file.open("r+") as fp:
            found = False
            i = 10  # assume that copyright is in the first 10 lines
            while not found and i > 0:
                line = fp.readline()
                if line == "":
                    i = 0
                    continue
                found, copyright_match = _find_copyright(
                    line=line, copyright_regex=copyright_regex
                )
                i = i - 1
            # header not found, add header
            if i == 0 and not found:
                try:
                    header = _add_header(
                        file.suffix,
                        license_id,
                        company,
                        year,
                    )
                    if header:
                        fp.seek(0)  # back to beginning of file
                        rest_of_file = fp.read()
                        fp.seek(0)
                        fp.write(header + "\n" + rest_of_file)
                        print(f"{file}: Added license header.")
                        return

                except ValueError:
                    print(
                        f"{file}: No license header for the"
                        f" format {file.suffix} found.",
                    )
                except FileNotFoundError:
                    print(
                        f"{file}: License file for {license_id} "
                        "is not existing."
                    )
                return

            # replace found header and write it to file
            if copyright_match:

                # use different target license formats depending on provided single_year argument
                if single_year:
                    copyright_term = (
                        f"SPDX-FileCopyrightText: "
                        f"{copyright_match.creation_year} "
                        f"{company}"
                    )
                else:
                    copyright_term = (
                        f"SPDX-FileCopyrightText: "
                        f"{copyright_match.creation_year}"
                        f"-{year} {company}"
                    )

                with_multi_year = (
                    copyright_match.creation_year
                    and copyright_match.modification_year
                )
                with_single_year_outdated = (
                    not copyright_match.modification_year
                    and int(copyright_match.creation_year) < int(year)
                )

                with_multi_year_outdated = False
                if with_multi_year:
                    # assert to silence mypy
                    assert isinstance(copyright_match.modification_year, str)
                    with_multi_year_outdated = int(
                        copyright_match.modification_year
                    ) < int(year)

                if single_year and with_multi_year:
                    _substitute_license_text(
                        fp, line, copyright_regex, copyright_term
                    )
                    print(
                        f"{file}: Changed License Header Copyright Year format to single year "
                        f"{copyright_match.creation_year}-{year} -> "
                        f"{copyright_match.creation_year}"
                    )
                elif not single_year and (
                    with_multi_year_outdated or with_single_year_outdated
                ):
                    _substitute_license_text(
                        fp, line, copyright_regex, copyright_term
                    )
                    print(
                        f"{file}: Changed License Header Copyright Year "
                        f"{copyright_match.modification_year} -> "
                        f"{year}"
                    )
                else:
                    print(f"{file}: License Header is ok.")

    except FileNotFoundError as e:
        print(f"{file}: File is not existing.")
        raise e
    except UnicodeDecodeError as e:
        print(f"{file}: Ignoring binary file.")
        raise e
    # old header existing - cleanup?
    if cleanup_regexes:
        old_content = file.read_text(encoding="utf-8")
        new_content = _remove_outdated_lines(
            content=old_content, cleanup_regexes=cleanup_regexes
        )
        if new_content:
            file.write_text(new_content, encoding="utf-8")
            print(f"{file}: Cleaned up!")


def _substitute_license_text(
    fp: io.TextIOWrapper,
    line: str,
    copyright_regex: re.Pattern,
    copyright_term: str,
) -> None:
    """Substitute the old license text in file fp, starting on provided line, with the new one provided in copyright_term"""
    new_line = re.sub(copyright_regex, copyright_term, line)
    fp_write = fp.tell() - len(line)  # save position to insert
    rest_of_file = fp.read()
    fp.seek(fp_write)
    fp.write(new_line)
    fp.write(rest_of_file)
    # in some cases we replace "YYYY - YYYY" with "YYYY-YYYY"
    # resulting in 2 characters left at the end of the file
    # so we truncate the file, just in case!
    fp.truncate()


def _get_exclude_list(
    exclude_file: Path, directories: list[Path]
) -> list[Path]:
    """Tries to get the list of excluded files / directories.
    If a file is given, it will be used. Otherwise it will be searched
    in the executed root path.
    The ignore file should only contain relative paths like *.py,
    not absolute as **/*.py
    """

    if exclude_file is None:
        exclude_file = Path(".pontos-header-ignore")

    if not exclude_file.is_file():
        return []

    exclude_lines = exclude_file.read_text(encoding="utf-8").splitlines()

    expanded_globs = [
        directory.rglob(line.strip())
        for directory in directories
        for line in exclude_lines
        if line
    ]

    exclude_list = []
    for glob_paths in expanded_globs:
        for path in glob_paths:
            if path.is_dir():
                for efile in path.rglob("*"):
                    exclude_list.append(efile.absolute())
            else:
                exclude_list.append(path.absolute())

    return exclude_list


@cache
def _compile_outdated_regex() -> list[re.Pattern]:
    """prepare regex patterns to remove old copyright lines"""
    return [re.compile(rf"^(([#*]|//) ?)?{line}") for line in OLD_LINES]


@cache
def _compile_copyright_regex() -> re.Pattern:
    """prepare the copyright regex"""
    c_str = r"(SPDX-FileCopyrightText:|[Cc]opyright)"
    d_str = r"(19[0-9]{2}|20[0-9]{2})"

    return re.compile(rf"{c_str}.*? {d_str}?-? ?{d_str}? (.+)")


def main(args: Optional[Sequence[str]] = None) -> None:
    parsed_args = parse_args(args)
    exclude_list = []
    year: str = parsed_args.year
    license_id: str = parsed_args.license_id
    company: str = parsed_args.company
    changed: bool = parsed_args.changed
    quiet: bool = parsed_args.quiet
    cleanup: bool = parsed_args.cleanup
    single_year: bool = parsed_args.single_year

    if quiet:
        term: Union[NullTerminal, RichTerminal] = NullTerminal()
    else:
        term = RichTerminal()

    term.bold_info("pontos-update-header")

    if parsed_args.directories:
        if isinstance(parsed_args.directories, list):
            directories = [
                Path(directory) for directory in parsed_args.directories
            ]
        else:
            directories = [Path(parsed_args.directories)]
        # get file paths to exclude
        exclude_list = _get_exclude_list(parsed_args.exclude_file, directories)
        # get files to update
        files = [
            Path(file)
            for directory in directories
            for file in directory.rglob("*")
            if file.is_file()
        ]
    elif parsed_args.files:
        if isinstance(parsed_args.files, list):
            files = [Path(name) for name in parsed_args.files]
        else:
            files = [Path(parsed_args.files)]

    else:
        # should never happen
        term.error("Specify files to update!")
        sys.exit(1)

    for file in files:

        try:
            if file.absolute() in exclude_list:
                term.warning(f"{file}: Ignoring file from exclusion list.")
            else:
                if changed:
                    try:
                        year = _get_modified_year(file)
                    except PontosError:
                        term.warning(
                            f"{file}: Could not get date of last modification"
                            f" via git, using {year} instead."
                        )

                update_file(
                    file,
                    year,
                    license_id,
                    company,
                    cleanup=cleanup,
                    single_year=single_year,
                )
        except (FileNotFoundError, UnicodeDecodeError, ValueError):
            continue


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa:E501

import unittest
from datetime import datetime, timezone

from pontos.github.models.secret_scanning import (
    AlertState,
    Resolution,
    SecretScanningAlert,
)

ALERT = {
    "number": 2,
    "created_at": "2020-11-06T18:48:51Z",
    "url": "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/2",
    "html_url": "https://github.com/owner/private-repo/security/secret-scanning/2",
    "locations_url": "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/2/locations",
    "state": "resolved",
    "resolution": "false_positive",
    "resolved_at": "2020-11-07T02:47:13Z",
    "resolved_by": {
        "login": "monalisa",
        "id": 2,
        "node_id": "MDQ6VXNlcjI=",
        "avatar_url": "https://alambic.github.com/avatars/u/2?",
        "gravatar_id": "",
        "url": "https://api.github.com/users/monalisa",
        "html_url": "https://github.com/monalisa",
        "followers_url": "https://api.github.com/users/monalisa/followers",
        "following_url": "https://api.github.com/users/monalisa/following{/other_user}",
        "gists_url": "https://api.github.com/users/monalisa/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/monalisa/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/monalisa/subscriptions",
        "organizations_url": "https://api.github.com/users/monalisa/orgs",
        "repos_url": "https://api.github.com/users/monalisa/repos",
        "events_url": "https://api.github.com/users/monalisa/events{/privacy}",
        "received_events_url": "https://api.github.com/users/monalisa/received_events",
        "type": "User",
        "site_admin": True,
    },
    "secret_type": "adafruit_io_key",
    "secret_type_display_name": "Adafruit IO Key",
    "secret": "aio_XXXXXXXXXXXXXXXXXXXXXXXXXXXX",
    "repository": {
        "id": 1296269,
        "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
        "name": "Hello-World",
        "full_name": "octocat/Hello-World",
        "owner": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "private": False,
        "html_url": "https://github.com/octocat/Hello-World",
        "description": "This your first repo!",
        "fork": False,
        "url": "https://api.github.com/repos/octocat/Hello-World",
        "archive_url": "https://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
        "assignees_url": "https://api.github.com/repos/octocat/Hello-World/assignees{/user}",
        "blobs_url": "https://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
        "branches_url": "https://api.github.com/repos/octocat/Hello-World/branches{/branch}",
        "collaborators_url": "https://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
        "comments_url": "https://api.github.com/repos/octocat/Hello-World/comments{/number}",
        "commits_url": "https://api.github.com/repos/octocat/Hello-World/commits{/sha}",
        "compare_url": "https://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
        "contents_url": "https://api.github.com/repos/octocat/Hello-World/contents/{+path}",
        "contributors_url": "https://api.github.com/repos/octocat/Hello-World/contributors",
        "deployments_url": "https://api.github.com/repos/octocat/Hello-World/deployments",
        "downloads_url": "https://api.github.com/repos/octocat/Hello-World/downloads",
        "events_url": "https://api.github.com/repos/octocat/Hello-World/events",
        "forks_url": "https://api.github.com/repos/octocat/Hello-World/forks",
        "git_commits_url": "https://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
        "git_refs_url": "https://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
        "git_tags_url": "https://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
        "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
        "issue_events_url": "https://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
        "issues_url": "https://api.github.com/repos/octocat/Hello-World/issues{/number}",
        "keys_url": "https://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
        "labels_url": "https://api.github.com/repos/octocat/Hello-World/labels{/name}",
        "languages_url": "https://api.github.com/repos/octocat/Hello-World/languages",
        "merges_url": "https://api.github.com/repos/octocat/Hello-World/merges",
        "milestones_url": "https://api.github.com/repos/octocat/Hello-World/milestones{/number}",
        "notifications_url": "https://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
        "pulls_url": "https://api.github.com/repos/octocat/Hello-World/pulls{/number}",
        "releases_url": "https://api.github.com/repos/octocat/Hello-World/releases{/id}",
        "stargazers_url": "https://api.github.com/repos/octocat/Hello-World/stargazers",
        "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
        "subscribers_url": "https://api.github.com/repos/octocat/Hello-World/subscribers",
        "subscription_url": "https://api.github.com/repos/octocat/Hello-World/subscription",
        "tags_url": "https://api.github.com/repos/octocat/Hello-World/tags",
        "teams_url": "https://api.github.com/repos/octocat/Hello-World/teams",
        "trees_url": "https://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
        "hooks_url": "https://api.github.com/repos/octocat/Hello-World/hooks",
    },
    "push_protection_bypassed_by": {
        "login": "monalisa",
        "id": 2,
        "node_id": "MDQ6VXNlcjI=",
        "avatar_url": "https://alambic.github.com/avatars/u/2?",
        "gravatar_id": "",
        "url": "https://api.github.com/users/monalisa",
        "html_url": "https://github.com/monalisa",
        "followers_url": "https://api.github.com/users/monalisa/followers",
        "following_url": "https://api.github.com/users/monalisa/following{/other_user}",
        "gists_url": "https://api.github.com/users/monalisa/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/monalisa/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/monalisa/subscriptions",
        "organizations_url": "https://api.github.com/users/monalisa/orgs",
        "repos_url": "https://api.github.com/users/monalisa/repos",
        "events_url": "https://api.github.com/users/monalisa/events{/privacy}",
        "received_events_url": "https://api.github.com/users/monalisa/received_events",
        "type": "User",
        "site_admin": True,
    },
    "push_protection_bypassed": True,
    "push_protection_bypassed_at": "2020-11-06T21:48:51Z",
    "resolution_comment": "Example comment",
}


class SecretScanningAlertTestCase(unittest.TestCase):
    def test_from_dict(self):
        alert = SecretScanningAlert.from_dict(ALERT)

        self.assertEqual(alert.number, 2)
        self.assertEqual(
            alert.url,
            "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/2",
        )
        self.assertEqual(
            alert.html_url,
            "https://github.com/owner/private-repo/security/secret-scanning/2",
        )
        self.assertEqual(
            alert.locations_url,
            "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/2/locations",
        )
        self.assertEqual(alert.state, AlertState.RESOLVED)
        self.assertEqual(alert.secret_type, "adafruit_io_key")
        self.assertEqual(alert.secret_type_display_name, "Adafruit IO Key")
        self.assertEqual(alert.secret, "aio_XXXXXXXXXXXXXXXXXXXXXXXXXXXX")
        self.assertEqual(alert.repository.id, 1296269)
        self.assertEqual(
            alert.created_at,
            datetime(2020, 11, 6, 18, 48, 51, tzinfo=timezone.utc),
        )
        self.assertIsNone(alert.updated_at)

        self.assertEqual(alert.resolution, Resolution.FALSE_POSITIVE)
        self.assertEqual(alert.resolution_comment, "Example comment")
        self.assertEqual(
            alert.resolved_at,
            datetime(2020, 11, 7, 2, 47, 13, tzinfo=timezone.utc),
        )
        self.assertEqual(alert.resolved_by.login, "monalisa")

        self.assertTrue(alert.push_protection_bypassed)
        self.assertEqual(alert.push_protection_bypassed_by.login, "monalisa")
        self.assertEqual(
            alert.push_protection_bypassed_at,
            datetime(2020, 11, 6, 21, 48, 51, tzinfo=timezone.utc),
        )



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa:E501

import unittest
from datetime import datetime, timezone

from pontos.github.models.user import (
    EmailInformation,
    SSHPublicKey,
    SSHPublicKeyExtended,
)


class SSHPublicKeyTestCase(unittest.TestCase):
    def test_from_dict(self):
        key = SSHPublicKey.from_dict({"id": 1, "key": "ssh-rsa AAA..."})

        self.assertEqual(key.id, 1)
        self.assertEqual(key.key, "ssh-rsa AAA...")


class SSHPublicKeyExtendedTestCase(unittest.TestCase):
    def test_from_dict(self):
        key = SSHPublicKeyExtended.from_dict(
            {
                "key": "2Sg8iYjAxxmI2LvUXpJjkYrMxURPc8r+dB7TJyvv1234",
                "id": 2,
                "url": "https://api.github.com/user/keys/2",
                "title": "ssh-rsa AAAAB3NzaC1yc2EAAA",
                "created_at": "2020-06-11T21:31:57Z",
                "verified": False,
                "read_only": False,
            }
        )

        self.assertEqual(key.id, 2)
        self.assertEqual(
            key.key, "2Sg8iYjAxxmI2LvUXpJjkYrMxURPc8r+dB7TJyvv1234"
        )
        self.assertEqual(key.url, "https://api.github.com/user/keys/2")
        self.assertEqual(key.title, "ssh-rsa AAAAB3NzaC1yc2EAAA")
        self.assertFalse(key.verified)
        self.assertFalse(key.read_only)
        self.assertEqual(
            key.created_at,
            datetime(2020, 6, 11, 21, 31, 57, tzinfo=timezone.utc),
        )


class EmailInformationTestCase(unittest.TestCase):
    def test_from_dict(self):
        email = EmailInformation.from_dict(
            {
                "email": "octocat@github.com",
                "verified": True,
                "primary": True,
                "visibility": "public",
            }
        )

        self.assertEqual(email.email, "octocat@github.com")
        self.assertEqual(email.visibility, "public")
        self.assertTrue(email.primary)
        self.assertTrue(email.verified)



# SPDX-FileCopyrightText: 2021-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from dataclasses import dataclass
from pathlib import Path
from unittest.mock import MagicMock, patch

from pontos.testing import temp_directory, temp_file
from pontos.version import VersionError
from pontos.version.commands._go import GoVersionCommand
from pontos.version.schemes import SemanticVersioningScheme


@dataclass
class StdOutput:
    stdout: bytes


VERSION_FILE_PATH = "version.go"
TEMPLATE = """package {}

// THIS IS AN AUTOGENERATED FILE. DO NOT TOUCH!

var version = "{}"
\n"""


class GetCurrentGoVersionCommandTestCase(unittest.TestCase):
    def test_getting_version(self):
        with temp_file(
            name="go.mod",
            change_into=True,
        ):
            version = "0.0.1"
            version_file_path = Path(VERSION_FILE_PATH)
            version_file_path.write_text(
                TEMPLATE.format("main", version), encoding="utf-8"
            )
            result_version = GoVersionCommand(
                SemanticVersioningScheme
            ).get_current_version()

            self.assertEqual(
                result_version, SemanticVersioningScheme.parse_version(version)
            )
            version_file_path.unlink()

    def test_no_version_found(self):
        exp_err_msg = "No version found in the version.go file."
        with (
            temp_file(
                name="go.mod",
                change_into=True,
            ),
            self.assertRaisesRegex(
                VersionError,
                exp_err_msg,
            ),
        ):
            version_file_path = Path(VERSION_FILE_PATH)
            version_file_path.touch()
            GoVersionCommand(SemanticVersioningScheme).get_current_version()

    def test_no_version_file(self):
        exp_err_msg = (
            "No version.go file found. This file is required for pontos"
        )
        with (
            temp_file(
                name="go.mod",
                change_into=True,
            ),
            self.assertRaisesRegex(
                VersionError,
                exp_err_msg,
            ),
        ):
            GoVersionCommand(SemanticVersioningScheme).get_current_version()

    def test_invalid_version(self):
        with (
            temp_file(
                name="go.mod",
                change_into=True,
            ),
            self.assertRaisesRegex(
                VersionError, "abc is not valid SemVer string"
            ),
        ):
            version_file_path = Path(VERSION_FILE_PATH)
            version_file_path.write_text(
                TEMPLATE.format("main", "abc"), encoding="utf-8"
            )
            GoVersionCommand(SemanticVersioningScheme).get_current_version()


class VerifyGoVersionCommandTestCase(unittest.TestCase):
    def test_verify_version(self):
        with (
            temp_file(
                name="go.mod",
                change_into=True,
            ),
            patch.object(
                GoVersionCommand,
                "get_current_version",
                MagicMock(
                    return_value=SemanticVersioningScheme.parse_version(
                        "21.0.1"
                    )
                ),
            ),
        ):
            cmd = GoVersionCommand(SemanticVersioningScheme)
            cmd.verify_version(SemanticVersioningScheme.parse_version("21.0.1"))

    def test_verify_branch_not_equal(self):
        with (
            temp_file(
                name="go.mod",
                change_into=True,
            ),
            patch.object(
                GoVersionCommand,
                "get_current_version",
                MagicMock(
                    return_value=SemanticVersioningScheme.parse_version(
                        "21.0.1"
                    )
                ),
            ),
            self.assertRaisesRegex(
                VersionError,
                "Provided version 21.2.0 does not match the current version "
                "21.0.1.",
            ),
        ):
            cmd = GoVersionCommand(SemanticVersioningScheme)
            cmd.verify_version(SemanticVersioningScheme.parse_version("21.2.0"))

    def test_verify_current(self):
        with (
            temp_file(
                name="go.mod",
                change_into=True,
            ),
            patch.object(
                GoVersionCommand,
                "get_current_version",
                MagicMock(
                    return_value=SemanticVersioningScheme.parse_version(
                        "21.0.1"
                    )
                ),
            ),
        ):
            cmd = GoVersionCommand(SemanticVersioningScheme)
            cmd.verify_version("current")
            cmd.verify_version(version=None)

    def test_verify_current_failure(self):
        with temp_file(
            name="go.mod",
            change_into=True,
        ):
            cmd = GoVersionCommand(SemanticVersioningScheme)

            with self.assertRaisesRegex(
                VersionError,
                "^No version.go file found. This file is required for pontos",
            ):
                cmd.verify_version("current")

            with self.assertRaisesRegex(
                VersionError,
                "^No version.go file found. This file is required for pontos",
            ):
                cmd.verify_version(version=None)


class UpdateGoVersionCommandTestCase(unittest.TestCase):
    def test_no_file_update_version(self):
        with temp_directory(change_into=True) as temp:
            go_mod = temp / "go.mod"
            go_mod.touch()

            version = SemanticVersioningScheme.parse_version("22.2.2")
            updated_version_obj = GoVersionCommand(
                SemanticVersioningScheme
            ).update_version(version)
            version_file_path = Path(VERSION_FILE_PATH)
            content = version_file_path.read_text(encoding="utf-8")

            self.assertIn(str(version), content)

            self.assertIsNone(updated_version_obj.previous)
            self.assertEqual(updated_version_obj.new, version)
            self.assertEqual(
                updated_version_obj.changed_files, [version_file_path]
            )

    def test_update_version(self):
        with temp_file(name="go.mod", change_into=True):
            cmd = GoVersionCommand(SemanticVersioningScheme)
            version = SemanticVersioningScheme.parse_version("22.2.2")
            version_file_path = Path(VERSION_FILE_PATH)
            version_file_path.write_text(
                TEMPLATE.format("foo", "0.0.1"), encoding="utf-8"
            )
            updated = cmd.update_version(version)

            content = version_file_path.read_text(encoding="utf-8")
            self.assertIn(str(version), content)
            self.assertIn("foo", content)
            version_file_path.unlink()

            self.assertEqual(updated.new, version)
            self.assertEqual(
                updated.previous,
                SemanticVersioningScheme.parse_version("0.0.1"),
            )
            self.assertEqual(updated.changed_files, [version_file_path])

    def test_create_file_update_version(self):
        with temp_file(name="go.mod", change_into=True):
            with patch.object(
                GoVersionCommand,
                "get_current_version",
                MagicMock(
                    return_value=SemanticVersioningScheme.parse_version(
                        "21.0.1"
                    )
                ),
            ):
                version = SemanticVersioningScheme.parse_version("22.2.2")
                cmd = GoVersionCommand(SemanticVersioningScheme)
                updated = cmd.update_version(version)

                version_file_path = Path(VERSION_FILE_PATH)
                content = version_file_path.read_text(encoding="utf-8")

                self.assertIn(str(version), content)
                version_file_path.unlink()

                self.assertEqual(updated.new, version)
                self.assertEqual(
                    updated.previous,
                    SemanticVersioningScheme.parse_version("21.0.1"),
                )
                self.assertEqual(updated.changed_files, [version_file_path])

    def test_no_update(self):
        with temp_file(name="go.mod", change_into=True):
            cmd = GoVersionCommand(SemanticVersioningScheme)
            version = SemanticVersioningScheme.parse_version("22.2.2")
            version_file_path = Path(VERSION_FILE_PATH)
            version_file_path.write_text(
                TEMPLATE.format("main", "22.2.2"), encoding="utf-8"
            )
            updated = cmd.update_version(version)

            content = version_file_path.read_text(encoding="utf-8")
            self.assertIn(str(version), content)

            self.assertEqual(updated.new, version)
            self.assertEqual(updated.previous, version)
            self.assertEqual(updated.changed_files, [])

    def test_forced_update(self):
        with temp_file(name="go.mod", change_into=True):
            cmd = GoVersionCommand(SemanticVersioningScheme)
            version = SemanticVersioningScheme.parse_version("22.2.2")
            version_file_path = Path(VERSION_FILE_PATH)
            version_file_path.write_text(
                TEMPLATE.format("main", "22.2.2"), encoding="utf-8"
            )
            updated = cmd.update_version(version, force=True)

            content = version_file_path.read_text(encoding="utf-8")
            self.assertIn(str(version), content)

            self.assertEqual(updated.new, version)
            self.assertEqual(updated.previous, version)
            self.assertEqual(updated.changed_files, [version_file_path])


class ProjectFileGoVersionCommandTestCase(unittest.TestCase):
    def test_project_file_not_found(self):
        with temp_directory(change_into=True):
            cmd = GoVersionCommand(SemanticVersioningScheme)

            self.assertFalse(cmd.project_found())

    def test_project_file_found(self):
        with temp_file(name="go.mod", change_into=True):
            cmd = GoVersionCommand(SemanticVersioningScheme)

            self.assertTrue(cmd.project_found())



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

"""
This script creates a new repository with default settings
"""

import shutil
from argparse import ArgumentParser, BooleanOptionalAction, Namespace
from typing import Union

from pontos.git import Git, MergeStrategy
from pontos.github.api import GitHubAsyncRESTApi
from pontos.github.api.repositories import GitIgnoreTemplate, LicenseType
from pontos.github.models.base import Permission
from pontos.github.script.errors import GitHubScriptError
from pontos.testing import temp_directory

TEMPLATES = {
    "python": "https://github.com/greenbone/python-project-template.git",
    "go": "https://github.com/greenbone/go-project-template.git",
}

GITIGNORE = {"python": GitIgnoreTemplate.PYTHON, "go": GitIgnoreTemplate.GO}


def license_type(value: Union[str, LicenseType]) -> LicenseType:
    if isinstance(value, LicenseType):
        return value

    return LicenseType(value.lower())


def possible_license_types() -> str:
    return ", ".join(
        [
            LicenseType.GNU_GENERAL_PUBLIC_LICENSE_2_0.value,
            LicenseType.GNU_GENERAL_PUBLIC_LICENSE_3_0.value,
            LicenseType.GNU_AFFERO_GENERAL_PUBLIC_LICENSE_3_0.value,
        ]
    )


def add_script_arguments(parser: ArgumentParser) -> None:
    parser.add_argument(
        "--template",
        choices=("python", "go"),
        help="Use template repo as base for the new repository.",
    )
    parser.add_argument(
        "--team",
        help="Team that should have admin access to the repository.",
    )
    parser.add_argument(
        "--license",
        help=f"License to choose for the repo: {possible_license_types()}. "
        "Default: %(default)s.",
        type=license_type,
        default=LicenseType.GNU_AFFERO_GENERAL_PUBLIC_LICENSE_3_0.value,
    )
    parser.add_argument(
        "--visibility",
        choices=("public", "private"),
        default="private",
        help="Visibility of the repository. Default: %(default)s.",
    )
    parser.add_argument("--description", help="Description of the repository.")
    parser.add_argument(
        "--branch-protection",
        action=BooleanOptionalAction,
        default=True,
        help="Enable/Disable branch protection for the main branch. Default is "
        "enabled.",
    )
    parser.add_argument("name", help="Repository to create.")
    parser.add_argument(
        "organization",
        nargs="?",
        default="greenbone",
        help="Organization to create the repo in. Default: %(default)s.",
    )


async def github_script(api: GitHubAsyncRESTApi, args: Namespace) -> int:
    organization = args.organization
    repository = args.name
    private = True if args.visibility == "private" else False
    gitignore_template = GITIGNORE.get(args.template)
    license_template = args.license
    description = args.description
    branch_protection = args.branch_protection

    if args.team:
        team = await api.teams.get(organization, args.team)
        team_id = team.id
    else:
        team_id = None
        team = None

    with temp_directory() as temp_dir:
        git = Git()

        if args.template:
            git.clone(TEMPLATES[args.template], temp_dir, remote="template")
            dot_git_dir = temp_dir / ".git"
            shutil.rmtree(dot_git_dir)

            git.cwd = temp_dir
            git.init()
            git.add(".")
            git.commit(f"Starting commit from {args.template} template")
        else:
            git.cwd = temp_dir
            git.init()

        repo = await api.repositories.create(
            organization,
            repository,
            private=private,
            has_projects=False,
            has_wiki=False,
            allow_merge_commit=True,
            allow_auto_merge=True,
            allow_rebase_merge=True,
            allow_squash_merge=True,
            allow_update_branch=True,
            delete_branch_on_merge=True,
            is_template=False,
            license_template=license_template,
            description=description,
            auto_init=True,
            gitignore_template=gitignore_template,
            team_id=team_id,
        )

        repo_url = repo.ssh_url

        if not repo_url:
            raise GitHubScriptError("No ssh repository URL")

        git.add_remote("upstream", repo_url)
        git.fetch("upstream")

        if args.template:
            git.rebase("upstream/main", strategy=MergeStrategy.ORT_OURS)
        else:
            git.checkout("main", start_point="upstream/main")

        if team:
            await api.teams.update_permission(
                organization, team.slug, repository, Permission.ADMIN
            )

            code_owners_file = temp_dir / ".github" / "CODEOWNERS"
            code_owners_file.write_text(
                f"# default reviewers\n*\t@{organization}/{team.slug}\n"
            )
            git.add(code_owners_file)
            git.commit("Adjust CODEOWNERS file")

        git.push(remote="upstream", force=True)

    if branch_protection:
        await api.branches.update_protection_rules(
            f"{organization}/{repository}",
            "main",
            require_branches_to_be_up_to_date=True,
            require_code_owner_reviews=True,
            required_approving_review_count=1,
            required_conversation_resolution=True,
            dismiss_stale_reviews=True,
            allow_force_pushes=False,
            allow_deletions=False,
            restrictions_users=[],
        )

    return 0



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

"""
This script locks a branch in a repo of an organization via branch protection
"""

from argparse import ArgumentParser, BooleanOptionalAction, Namespace

from pontos.github.api import GitHubAsyncRESTApi
from pontos.github.api.branch import update_from_applied_settings


def add_script_arguments(parser: ArgumentParser) -> None:
    parser.add_argument("repository", help="org/repo combination")
    parser.add_argument("branch", help="branch to lock")
    parser.add_argument("--lock", action=BooleanOptionalAction, default=True)


async def github_script(api: GitHubAsyncRESTApi, args: Namespace) -> int:
    branch_protection = await api.branches.protection_rules(
        args.repository, args.branch
    )
    kwargs = update_from_applied_settings(
        branch_protection,
        lock_branch=args.lock,
    )
    await api.branches.update_protection_rules(
        args.repository,
        args.branch,
        **kwargs,
    )
    if args.lock:
        print(f"Locked branch {args.branch} in {args.repository}")
    else:
        print(f"Unlocked branch {args.branch} in {args.repository}")
    return 0



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa:E501

import unittest

from pontos.github.models.billing import (
    ActionsBilling,
    PackagesBilling,
    StorageBilling,
)


class ActionsBillingTestCase(unittest.TestCase):
    def test_from_dict(self):
        billing = ActionsBilling.from_dict(
            {
                "total_minutes_used": 305,
                "total_paid_minutes_used": 0,
                "included_minutes": 3000,
                "minutes_used_breakdown": {
                    "UBUNTU": 205,
                    "MACOS": 10,
                    "WINDOWS": 90,
                },
            }
        )

        self.assertEqual(billing.total_minutes_used, 305)
        self.assertEqual(billing.total_paid_minutes_used, 0)
        self.assertEqual(billing.included_minutes, 3000)
        self.assertEqual(billing.minutes_used_breakdown.UBUNTU, 205)
        self.assertEqual(billing.minutes_used_breakdown.MACOS, 10)
        self.assertEqual(billing.minutes_used_breakdown.WINDOWS, 90)
        self.assertIsNone(billing.minutes_used_breakdown.total)


class PackagesBillingTestCase(unittest.TestCase):
    def test_from_dict(self):
        billing = PackagesBilling.from_dict(
            {
                "total_gigabytes_bandwidth_used": 50,
                "total_paid_gigabytes_bandwidth_used": 40,
                "included_gigabytes_bandwidth": 10,
            }
        )

        self.assertEqual(billing.total_gigabytes_bandwidth_used, 50)
        self.assertEqual(billing.total_paid_gigabytes_bandwidth_used, 40)
        self.assertEqual(billing.included_gigabytes_bandwidth, 10)


class StorageBillingTestCase(unittest.TestCase):
    def test_from_dict(self):
        billing = StorageBilling.from_dict(
            {
                "days_left_in_billing_cycle": 20,
                "estimated_paid_storage_for_month": 15,
                "estimated_storage_for_month": 40,
            }
        )
        self.assertEqual(billing.days_left_in_billing_cycle, 20)
        self.assertEqual(billing.estimated_paid_storage_for_month, 15)
        self.assertEqual(billing.estimated_storage_for_month, 40)



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa:E501

import json

from pontos.github.api.code_scanning import GitHubAsyncRESTCodeScanning
from pontos.github.models.base import SortOrder
from pontos.github.models.code_scanning import (
    AlertSort,
    AlertState,
    DefaultSetupState,
    DismissedReason,
    Language,
    QuerySuite,
    SarifProcessingStatus,
    Severity,
)
from tests import AsyncIteratorMock, aiter, anext
from tests.github.api import GitHubAsyncRESTTestCase, create_response

ALERTS = [
    {
        "number": 4,
        "created_at": "2020-02-13T12:29:18Z",
        "url": "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/4",
        "html_url": "https://github.com/octocat/hello-world/code-scanning/4",
        "state": "open",
        "dismissed_by": None,
        "dismissed_at": None,
        "dismissed_reason": None,
        "dismissed_comment": None,
        "rule": {
            "id": "js/zipslip",
            "severity": "error",
            "tags": ["security", "external/cwe/cwe-022"],
            "description": "Arbitrary file write during zip extraction",
            "name": "js/zipslip",
        },
        "tool": {"name": "CodeQL", "guid": None, "version": "2.4.0"},
        "most_recent_instance": {
            "ref": "refs/heads/main",
            "analysis_key": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
            "category": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
            "environment": "{}",
            "state": "open",
            "commit_sha": "39406e42cb832f683daa691dd652a8dc36ee8930",
            "message": {"text": "This path depends on a user-provided value."},
            "location": {
                "path": "spec-main/api-session-spec.ts",
                "start_line": 917,
                "end_line": 917,
                "start_column": 7,
                "end_column": 18,
            },
            "classifications": ["test"],
        },
        "instances_url": "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/4/instances",
        "repository": {
            "id": 1296269,
            "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
            "name": "Hello-World",
            "full_name": "octocat/Hello-World",
            "owner": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "private": False,
            "html_url": "https://github.com/octocat/Hello-World",
            "description": "This your first repo!",
            "fork": False,
            "url": "https://api.github.com/repos/octocat/Hello-World",
            "archive_url": "https://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
            "assignees_url": "https://api.github.com/repos/octocat/Hello-World/assignees{/user}",
            "blobs_url": "https://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
            "branches_url": "https://api.github.com/repos/octocat/Hello-World/branches{/branch}",
            "collaborators_url": "https://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
            "comments_url": "https://api.github.com/repos/octocat/Hello-World/comments{/number}",
            "commits_url": "https://api.github.com/repos/octocat/Hello-World/commits{/sha}",
            "compare_url": "https://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
            "contents_url": "https://api.github.com/repos/octocat/Hello-World/contents/{+path}",
            "contributors_url": "https://api.github.com/repos/octocat/Hello-World/contributors",
            "deployments_url": "https://api.github.com/repos/octocat/Hello-World/deployments",
            "downloads_url": "https://api.github.com/repos/octocat/Hello-World/downloads",
            "events_url": "https://api.github.com/repos/octocat/Hello-World/events",
            "forks_url": "https://api.github.com/repos/octocat/Hello-World/forks",
            "git_commits_url": "https://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
            "git_refs_url": "https://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
            "git_tags_url": "https://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
            "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
            "issue_events_url": "https://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
            "issues_url": "https://api.github.com/repos/octocat/Hello-World/issues{/number}",
            "keys_url": "https://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
            "labels_url": "https://api.github.com/repos/octocat/Hello-World/labels{/name}",
            "languages_url": "https://api.github.com/repos/octocat/Hello-World/languages",
            "merges_url": "https://api.github.com/repos/octocat/Hello-World/merges",
            "milestones_url": "https://api.github.com/repos/octocat/Hello-World/milestones{/number}",
            "notifications_url": "https://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
            "pulls_url": "https://api.github.com/repos/octocat/Hello-World/pulls{/number}",
            "releases_url": "https://api.github.com/repos/octocat/Hello-World/releases{/id}",
            "stargazers_url": "https://api.github.com/repos/octocat/Hello-World/stargazers",
            "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
            "subscribers_url": "https://api.github.com/repos/octocat/Hello-World/subscribers",
            "subscription_url": "https://api.github.com/repos/octocat/Hello-World/subscription",
            "tags_url": "https://api.github.com/repos/octocat/Hello-World/tags",
            "teams_url": "https://api.github.com/repos/octocat/Hello-World/teams",
            "trees_url": "https://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
            "hooks_url": "https://api.github.com/repos/octocat/Hello-World/hooks",
        },
    },
    {
        "number": 3,
        "created_at": "2020-02-13T12:29:18Z",
        "url": "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/3",
        "html_url": "https://github.com/octocat/hello-world/code-scanning/3",
        "state": "dismissed",
        "dismissed_by": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "dismissed_at": "2020-02-14T12:29:18Z",
        "dismissed_reason": "false positive",
        "dismissed_comment": "This alert is not actually correct, because there's a sanitizer included in the library.",
        "rule": {
            "id": "js/zipslip",
            "severity": "error",
            "tags": ["security", "external/cwe/cwe-022"],
            "description": "Arbitrary file write during zip extraction",
            "name": "js/zipslip",
        },
        "tool": {"name": "CodeQL", "guid": None, "version": "2.4.0"},
        "most_recent_instance": {
            "ref": "refs/heads/main",
            "analysis_key": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
            "category": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
            "environment": "{}",
            "state": "open",
            "commit_sha": "39406e42cb832f683daa691dd652a8dc36ee8930",
            "message": {"text": "This path depends on a user-provided value."},
            "location": {
                "path": "lib/ab12-gen.js",
                "start_line": 917,
                "end_line": 917,
                "start_column": 7,
                "end_column": 18,
            },
            "classifications": [],
        },
        "instances_url": "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/3/instances",
        "repository": {
            "id": 1296269,
            "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
            "name": "Hello-World",
            "full_name": "octocat/Hello-World",
            "owner": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "private": False,
            "html_url": "https://github.com/octocat/Hello-World",
            "description": "This your first repo!",
            "fork": False,
            "url": "https://api.github.com/repos/octocat/Hello-World",
            "archive_url": "https://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
            "assignees_url": "https://api.github.com/repos/octocat/Hello-World/assignees{/user}",
            "blobs_url": "https://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
            "branches_url": "https://api.github.com/repos/octocat/Hello-World/branches{/branch}",
            "collaborators_url": "https://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
            "comments_url": "https://api.github.com/repos/octocat/Hello-World/comments{/number}",
            "commits_url": "https://api.github.com/repos/octocat/Hello-World/commits{/sha}",
            "compare_url": "https://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
            "contents_url": "https://api.github.com/repos/octocat/Hello-World/contents/{+path}",
            "contributors_url": "https://api.github.com/repos/octocat/Hello-World/contributors",
            "deployments_url": "https://api.github.com/repos/octocat/Hello-World/deployments",
            "downloads_url": "https://api.github.com/repos/octocat/Hello-World/downloads",
            "events_url": "https://api.github.com/repos/octocat/Hello-World/events",
            "forks_url": "https://api.github.com/repos/octocat/Hello-World/forks",
            "git_commits_url": "https://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
            "git_refs_url": "https://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
            "git_tags_url": "https://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
            "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
            "issue_events_url": "https://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
            "issues_url": "https://api.github.com/repos/octocat/Hello-World/issues{/number}",
            "keys_url": "https://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
            "labels_url": "https://api.github.com/repos/octocat/Hello-World/labels{/name}",
            "languages_url": "https://api.github.com/repos/octocat/Hello-World/languages",
            "merges_url": "https://api.github.com/repos/octocat/Hello-World/merges",
            "milestones_url": "https://api.github.com/repos/octocat/Hello-World/milestones{/number}",
            "notifications_url": "https://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
            "pulls_url": "https://api.github.com/repos/octocat/Hello-World/pulls{/number}",
            "releases_url": "https://api.github.com/repos/octocat/Hello-World/releases{/id}",
            "stargazers_url": "https://api.github.com/repos/octocat/Hello-World/stargazers",
            "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
            "subscribers_url": "https://api.github.com/repos/octocat/Hello-World/subscribers",
            "subscription_url": "https://api.github.com/repos/octocat/Hello-World/subscription",
            "tags_url": "https://api.github.com/repos/octocat/Hello-World/tags",
            "teams_url": "https://api.github.com/repos/octocat/Hello-World/teams",
            "trees_url": "https://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
            "hooks_url": "https://api.github.com/repos/octocat/Hello-World/hooks",
        },
    },
]

ANALYSES = [
    {
        "ref": "refs/heads/main",
        "commit_sha": "d99612c3e1f2970085cfbaeadf8f010ef69bad83",
        "analysis_key": ".github/workflows/codeql-analysis.yml:analyze",
        "environment": '{"language":"python"}',
        "error": "",
        "category": ".github/workflows/codeql-analysis.yml:analyze/language:python",
        "created_at": "2020-08-27T15:05:21Z",
        "results_count": 17,
        "rules_count": 49,
        "id": 201,
        "url": "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses/201",
        "sarif_id": "6c81cd8e-b078-4ac3-a3be-1dad7dbd0b53",
        "tool": {"name": "CodeQL", "guid": None, "version": "2.4.0"},
        "deletable": True,
        "warning": "",
    },
    {
        "ref": "refs/heads/my-branch",
        "commit_sha": "c8cff6510d4d084fb1b4aa13b64b97ca12b07321",
        "analysis_key": ".github/workflows/shiftleft.yml:build",
        "environment": "{}",
        "error": "",
        "category": ".github/workflows/shiftleft.yml:build/",
        "created_at": "2020-08-31T22:46:44Z",
        "results_count": 17,
        "rules_count": 32,
        "id": 200,
        "url": "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses/200",
        "sarif_id": "8981cd8e-b078-4ac3-a3be-1dad7dbd0b582",
        "tool": {
            "name": "Python Security Analysis",
            "guid": None,
            "version": "1.2.0",
        },
        "deletable": True,
        "warning": "",
    },
]


class GitHubAsyncRESTCodeScanningTestCase(GitHubAsyncRESTTestCase):
    api_cls = GitHubAsyncRESTCodeScanning

    async def test_organization_alerts(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.organization_alerts("foo"))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/code-scanning/alerts",
            params={"per_page": "100", "sort": "created", "direction": "desc"},
        )

    async def test_organization_alerts_state(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.organization_alerts("foo", state=AlertState.FIXED)
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "state": "fixed",
            },
        )

    async def test_organization_alerts_tool_name(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.organization_alerts("foo", tool_name="CodeQL")
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "tool_name": "CodeQL",
            },
        )

    async def test_organization_alerts_tool_guid(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.organization_alerts("foo", tool_guid=None))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "tool_guid": None,
            },
        )

    async def test_organization_alerts_severity(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.organization_alerts("foo", severity=Severity.ERROR)
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "severity": "error",
            },
        )

    async def test_organization_alerts_sort(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.organization_alerts("foo", sort=AlertSort.UPDATED)
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "updated",
                "direction": "desc",
            },
        )

    async def test_organization_alerts_direction(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.organization_alerts("foo", direction=SortOrder.ASC)
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "asc",
            },
        )

    async def test_alerts(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar"))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/alerts",
            params={"per_page": "100", "sort": "created", "direction": "desc"},
        )

    async def test_alerts_state(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar", state=AlertState.FIXED))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "state": "fixed",
            },
        )

    async def test_alerts_tool_name(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar", tool_name="CodeQL"))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "tool_name": "CodeQL",
            },
        )

    async def test_alerts_tool_guid(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar", tool_guid=None))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "tool_guid": None,
            },
        )

    async def test_alerts_severity(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar", severity=Severity.ERROR))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "severity": "error",
            },
        )

    async def test_alerts_sort(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar", sort=AlertSort.UPDATED))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "updated",
                "direction": "desc",
            },
        )

    async def test_alerts_direction(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar", direction=SortOrder.ASC))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 4)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "asc",
            },
        )

    async def test_alert(self):
        response = create_response()
        response.json.return_value = {
            "number": 42,
            "created_at": "2020-06-19T11:21:34Z",
            "url": "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/42",
            "html_url": "https://github.com/octocat/hello-world/code-scanning/42",
            "state": "dismissed",
            "fixed_at": None,
            "dismissed_by": {
                "login": "octocat",
                "id": 54933897,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "dismissed_at": "2020-02-14T12:29:18Z",
            "dismissed_reason": "false positive",
            "dismissed_comment": "This alert is not actually correct, because there's a sanitizer included in the library.",
            "rule": {
                "id": "js/zipslip",
                "severity": "error",
                "security_severity_level": "high",
                "description": 'Arbitrary file write during zip extraction ("Zip Slip")',
                "name": "js/zipslip",
                "full_description": "Extracting files from a malicious zip archive without validating that the destination file path is within the destination directory can cause files outside the destination directory to be overwritten.",
                "tags": ["security", "external/cwe/cwe-022"],
                "help": '# Arbitrary file write during zip extraction ("Zip Slip")\\nExtracting files from a malicious zip archive without validating that the destination file path is within the destination directory can cause files outside the destination directory to be overwritten ...',
                "help_uri": "https://codeql.github.com/",
            },
            "tool": {"name": "CodeQL", "guid": None, "version": "2.4.0"},
            "most_recent_instance": {
                "ref": "refs/heads/main",
                "analysis_key": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
                "category": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
                "environment": "{}",
                "state": "dismissed",
                "commit_sha": "39406e42cb832f683daa691dd652a8dc36ee8930",
                "message": {
                    "text": "This path depends on a user-provided value."
                },
                "location": {
                    "path": "spec-main/api-session-spec.ts",
                    "start_line": 917,
                    "end_line": 917,
                    "start_column": 7,
                    "end_column": 18,
                },
                "classifications": ["test"],
            },
            "instances_url": "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/42/instances",
        }
        self.client.get.return_value = response

        alert = await self.api.alert(
            "foo/bar",
            42,
        )

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/code-scanning/alerts/42",
        )

        self.assertEqual(alert.number, 42)

    async def test_update(self):
        response = create_response()
        response.json.return_value = {
            "number": 42,
            "created_at": "2020-08-25T21:28:36Z",
            "url": "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/42",
            "html_url": "https://github.com/octocat/hello-world/code-scanning/42",
            "state": "dismissed",
            "fixed_at": None,
            "dismissed_by": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "dismissed_at": "2020-09-02T22:34:56Z",
            "dismissed_reason": "false positive",
            "dismissed_comment": "This alert is not actually correct, because there's a sanitizer included in the library.",
            "rule": {
                "id": "js/zipslip",
                "severity": "error",
                "security_severity_level": "high",
                "description": 'Arbitrary file write during zip extraction ("Zip Slip")',
                "name": "js/zipslip",
                "full_description": "Extracting files from a malicious zip archive without validating that the destination file path is within the destination directory can cause files outside the destination directory to be overwritten.",
                "tags": ["security", "external/cwe/cwe-022"],
                "help": '# Arbitrary file write during zip extraction ("Zip Slip")\\nExtracting files from a malicious zip archive without validating that the destination file path is within the destination directory can cause files outside the destination directory to be overwritten ...',
                "help_uri": "https://codeql.github.com/",
            },
            "tool": {"name": "CodeQL", "guid": None, "version": "2.4.0"},
            "most_recent_instance": {
                "ref": "refs/heads/main",
                "analysis_key": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
                "category": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
                "environment": "{}",
                "state": "dismissed",
                "commit_sha": "39406e42cb832f683daa691dd652a8dc36ee8930",
                "message": {
                    "text": "This path depends on a user-provided value."
                },
                "location": {
                    "path": "spec-main/api-session-spec.ts",
                    "start_line": 917,
                    "end_line": 917,
                    "start_column": 7,
                    "end_column": 18,
                },
                "classifications": ["test"],
            },
            "instances_url": "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/42/instances",
        }
        self.client.patch.return_value = response

        alert = await self.api.update_alert(
            "foo/bar",
            42,
            AlertState.DISMISSED,
            dismissed_reason=DismissedReason.USED_IN_TESTS,
            dismissed_comment="Only used in tests",
        )

        self.client.patch.assert_awaited_once_with(
            "/repos/foo/bar/code-scanning/alerts/42",
            data={
                "state": "dismissed",
                "dismissed_reason": "used in tests",
                "dismissed_comment": "Only used in tests",
            },
        )

        self.assertEqual(alert.number, 42)
        self.assertIsNone(alert.repository)

    async def test_alerts_instances(self):
        response = create_response()
        response.json.return_value = [
            {
                "ref": "refs/heads/main",
                "analysis_key": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
                "environment": "",
                "category": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
                "state": "open",
                "fixed_at": None,
                "commit_sha": "39406e42cb832f683daa691dd652a8dc36ee8930",
                "message": {
                    "text": "This path depends on a user-provided value."
                },
                "location": {
                    "path": "lib/ab12-gen.js",
                    "start_line": 917,
                    "end_line": 917,
                    "start_column": 7,
                    "end_column": 18,
                },
                "classifications": ["library"],
            },
            {
                "ref": "refs/pull/3740/merge",
                "analysis_key": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
                "environment": "",
                "category": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
                "state": "fixed",
                "fixed_at": "2020-02-14T12:29:18Z",
                "commit_sha": "b09da05606e27f463a2b49287684b4ae777092f2",
                "message": {
                    "text": "This suffix check is missing a length comparison to correctly handle lastIndexOf returning -1."
                },
                "location": {
                    "path": "app/script.js",
                    "start_line": 2,
                    "end_line": 2,
                    "start_column": 10,
                    "end_column": 50,
                },
                "classifications": ["source"],
            },
        ]

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.instances("foo/bar", 1))
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/main")
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/pull/3740/merge")

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/alerts/1/instances",
            params={
                "per_page": "100",
            },
        )

    async def test_analyses(self):
        response = create_response()
        response.json.return_value = ANALYSES

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.analyses("foo/bar"))
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/main")
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/my-branch")

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/analyses",
            params={
                "per_page": "100",
                "direction": "desc",
            },
        )

    async def test_analyses_tool_name(self):
        response = create_response()
        response.json.return_value = ANALYSES

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.analyses("foo/bar", tool_name="CodeQL"))
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/main")
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/my-branch")

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/analyses",
            params={
                "per_page": "100",
                "direction": "desc",
                "tool_name": "CodeQL",
            },
        )

    async def test_analyses_tool_guid(self):
        response = create_response()
        response.json.return_value = ANALYSES

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.analyses("foo/bar", tool_guid="123"))
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/main")
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/my-branch")

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/analyses",
            params={
                "per_page": "100",
                "direction": "desc",
                "tool_guid": "123",
            },
        )

    async def test_analyses_sarif_id(self):
        response = create_response()
        response.json.return_value = ANALYSES

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.analyses("foo/bar", sarif_id="123"))
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/main")
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/my-branch")

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/analyses",
            params={
                "per_page": "100",
                "direction": "desc",
                "sarif_id": "123",
            },
        )

    async def test_analyses_ref(self):
        response = create_response()
        response.json.return_value = ANALYSES

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.analyses("foo/bar", ref="refs/heads/main"))
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/main")
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/my-branch")

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/analyses",
            params={
                "per_page": "100",
                "direction": "desc",
                "ref": "refs/heads/main",
            },
        )

    async def test_analyses_direction(self):
        response = create_response()
        response.json.return_value = ANALYSES

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.analyses("foo/bar", direction=SortOrder.ASC))
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/main")
        instance = await anext(async_it)
        self.assertEqual(instance.ref, "refs/heads/my-branch")

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/analyses",
            params={
                "per_page": "100",
                "direction": "asc",
            },
        )

    async def test_analysis(self):
        response = create_response()
        response.json.return_value = {
            "ref": "refs/heads/main",
            "commit_sha": "c18c69115654ff0166991962832dc2bd7756e655",
            "analysis_key": ".github/workflows/codeql-analysis.yml:analyze",
            "environment": '{"language":"javascript"}',
            "error": "",
            "category": ".github/workflows/codeql-analysis.yml:analyze/language:javascript",
            "created_at": "2021-01-13T11:55:49Z",
            "results_count": 3,
            "rules_count": 67,
            "id": 3602840,
            "url": "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses/201",
            "sarif_id": "47177e22-5596-11eb-80a1-c1e54ef945c6",
            "tool": {"name": "CodeQL", "guid": None, "version": "2.4.0"},
            "deletable": True,
            "warning": "",
        }
        self.client.get.return_value = response

        alert = await self.api.analysis(
            "foo/bar",
            42,
        )

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/code-scanning/analyses/42",
        )

        self.assertEqual(alert.ref, "refs/heads/main")

    async def test_delete_analysis(self):
        response = create_response()
        response.json.return_value = {
            "next_analysis_url": "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses/41",
            "confirm_delete_url": "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses/41?confirm_delete",
        }
        self.client.delete.return_value = response

        resp = await self.api.delete_analysis(
            "foo/bar",
            42,
        )

        self.client.delete.assert_awaited_once_with(
            "/repos/foo/bar/code-scanning/analyses/42",
        )

        self.assertEqual(
            resp["next_analysis_url"],
            "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses/41",
        )
        self.assertEqual(
            resp["confirm_delete_url"],
            "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses/41?confirm_delete",
        )

    async def test_codeql_databases(self):
        response = create_response()
        response.json.return_value = [
            {
                "id": 1,
                "name": "database.zip",
                "language": "java",
                "uploader": {
                    "login": "octocat",
                    "id": 1,
                    "node_id": "MDQ6VXNlcjE=",
                    "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                    "gravatar_id": "",
                    "url": "https://api.github.com/users/octocat",
                    "html_url": "https://github.com/octocat",
                    "followers_url": "https://api.github.com/users/octocat/followers",
                    "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                    "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                    "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                    "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                    "organizations_url": "https://api.github.com/users/octocat/orgs",
                    "repos_url": "https://api.github.com/users/octocat/repos",
                    "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                    "received_events_url": "https://api.github.com/users/octocat/received_events",
                    "type": "User",
                    "site_admin": False,
                },
                "content_type": "application/zip",
                "size": 1024,
                "created_at": "2022-09-12T12:14:32Z",
                "updated_at": "2022-09-12T12:14:32Z",
                "url": "https://api.github.com/repos/octocat/Hello-World/code-scanning/codeql/databases/java",
                "commit_oid": 12345678901234567000,
            },
            {
                "id": 2,
                "name": "database.zip",
                "language": "ruby",
                "uploader": {
                    "login": "octocat",
                    "id": 1,
                    "node_id": "MDQ6VXNlcjE=",
                    "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                    "gravatar_id": "",
                    "url": "https://api.github.com/users/octocat",
                    "html_url": "https://github.com/octocat",
                    "followers_url": "https://api.github.com/users/octocat/followers",
                    "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                    "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                    "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                    "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                    "organizations_url": "https://api.github.com/users/octocat/orgs",
                    "repos_url": "https://api.github.com/users/octocat/repos",
                    "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                    "received_events_url": "https://api.github.com/users/octocat/received_events",
                    "type": "User",
                    "site_admin": False,
                },
                "content_type": "application/zip",
                "size": 1024,
                "created_at": "2022-09-12T12:14:32Z",
                "updated_at": "2022-09-12T12:14:32Z",
                "url": "https://api.github.com/repos/octocat/Hello-World/code-scanning/codeql/databases/ruby",
                "commit_oid": 23456789012345680000,
            },
        ]

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.codeql_databases("foo/bar"))
        db = await anext(async_it)
        self.assertEqual(db.id, 1)
        alert = await anext(async_it)
        self.assertEqual(alert.id, 2)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/code-scanning/codeql/databases",
            params={
                "per_page": "100",
            },
        )

    async def test_codeql_database(self):
        response = create_response()
        response.json.return_value = {
            "id": 1,
            "name": "database.zip",
            "language": "java",
            "uploader": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "content_type": "application/zip",
            "size": 1024,
            "created_at": "2022-09-12T12:14:32Z",
            "updated_at": "2022-09-12T12:14:32Z",
            "url": "https://api.github.com/repos/octocat/Hello-World/code-scanning/codeql/databases/java",
            "commit_oid": 12345678901234567000,
        }
        self.client.get.return_value = response

        alert = await self.api.codeql_database(
            "foo/bar",
            "java",
        )

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/code-scanning/codeql/databases/java",
        )

        self.assertEqual(alert.id, 1)

    async def test_default_setup(self):
        response = create_response()
        response.json.return_value = {
            "state": "configured",
            "languages": ["ruby", "python"],
            "query_suite": "default",
            "updated_at": "2023-01-19T11:21:34Z",
            "schedule": "weekly",
        }
        self.client.get.return_value = response

        setup = await self.api.default_setup(
            "foo/bar",
        )

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/code-scanning/default-setup",
        )

        self.assertEqual(setup.state, DefaultSetupState.CONFIGURED)

    async def test_update_default_setup(self):
        response = create_response()
        response.json.return_value = {
            "run_id": 42,
            "run_url": "https://api.github.com/repos/octoorg/octocat/actions/runs/42",
        }
        self.client.patch.return_value = response

        resp = await self.api.update_default_setup(
            "foo/bar",
            state=DefaultSetupState.CONFIGURED,
            query_suite=QuerySuite.EXTENDED,
            languages=[Language.GO],
        )

        self.client.patch.assert_awaited_once_with(
            "/repos/foo/bar/code-scanning/code-scanning/default-setup",
            data={
                "state": "configured",
                "query_suite": "extended",
                "languages": ["go"],
            },
        )

        self.assertEqual(
            resp["run_id"],
            42,
        )
        self.assertEqual(
            resp["run_url"],
            "https://api.github.com/repos/octoorg/octocat/actions/runs/42",
        )

    async def test_upload_sarif_data(self):
        sarif = {
            "version": "2.1.0",
            "$schema": "http://json.schemastore.org/sarif-2.1.0-rtm.4",
            "runs": [
                {
                    "tool": {
                        "driver": {
                            "name": "ESLint",
                            "informationUri": "https://eslint.org",
                            "rules": [
                                {
                                    "id": "no-unused-vars",
                                    "shortDescription": {
                                        "text": "disallow unused variables"
                                    },
                                    "helpUri": "https://eslint.org/docs/rules/no-unused-vars",
                                    "properties": {"category": "Variables"},
                                }
                            ],
                        }
                    },
                    "artifacts": [
                        {
                            "location": {
                                "uri": "file:///C:/dev/sarif/sarif-tutorials/samples/Introduction/simple-example.js"
                            }
                        }
                    ],
                    "results": [
                        {
                            "level": "error",
                            "message": {
                                "text": "'x' is assigned a value but never used."
                            },
                            "locations": [
                                {
                                    "physicalLocation": {
                                        "artifactLocation": {
                                            "uri": "file:///C:/dev/sarif/sarif-tutorials/samples/Introduction/simple-example.js",
                                            "index": 0,
                                        },
                                        "region": {
                                            "startLine": 1,
                                            "startColumn": 5,
                                        },
                                    }
                                }
                            ],
                            "ruleId": "no-unused-vars",
                            "ruleIndex": 0,
                        }
                    ],
                }
            ],
        }

        response = create_response()
        response.json.return_value = {
            "id": "47177e22-5596-11eb-80a1-c1e54ef945c6",
            "url": "https://api.github.com/repos/octocat/hello-world/code-scanning/sarifs/47177e22-5596-11eb-80a1-c1e54ef945c6",
        }
        self.client.post.return_value = response

        resp = await self.api.upload_sarif_data(
            "foo/bar",
            commit_sha="4b6472266afd7b471e86085a6659e8c7f2b119da",
            ref="refs/heads/master",
            sarif=json.dumps(sarif).encode(),
        )

        self.assertEqual(self.client.post.await_count, 1)
        args = self.client.post.await_args
        self.assertEqual(args.args, ("/repos/foo/bar/code-scanning/sarifs",))
        data = args.kwargs["data"]
        self.assertEqual(
            data["commit_sha"],
            "4b6472266afd7b471e86085a6659e8c7f2b119da",
        )
        self.assertEqual(
            data["ref"],
            "refs/heads/master",
        )
        # it's not possible to check the sarif data in Python < 3.11 because
        # gzip creates different content on each run
        self.assertTrue("sarif" in data)

        self.assertEqual(resp["id"], "47177e22-5596-11eb-80a1-c1e54ef945c6")
        self.assertEqual(
            resp["url"],
            "https://api.github.com/repos/octocat/hello-world/code-scanning/sarifs/47177e22-5596-11eb-80a1-c1e54ef945c6",
        )

    async def test_sarif(self):
        response = create_response()
        response.json.return_value = {
            "processing_status": "complete",
            "analyses_url": "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses?sarif_id=47177e22-5596-11eb-80a1-c1e54ef945c6",
        }
        self.client.get.return_value = response

        resp = await self.api.sarif(
            "foo/bar", "47177e22-5596-11eb-80a1-c1e54ef945c6"
        )
        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/code-scanning/sarifs/47177e22-5596-11eb-80a1-c1e54ef945c6",
        )

        self.assertEqual(resp.processing_status, SarifProcessingStatus.COMPLETE)
        self.assertEqual(
            resp.analyses_url,
            "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses?sarif_id=47177e22-5596-11eb-80a1-c1e54ef945c6",
        )
        self.assertIsNone(resp.errors)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=line-too-long

import unittest

from pontos.github.api.teams import TeamPrivacy
from pontos.github.models.base import Permission
from pontos.github.models.branch import (
    BranchProtection,
    RequiredPullRequestReviews,
    RequiredStatusChecks,
    Restrictions,
)


class RestrictionsTestCase(unittest.TestCase):
    def test_from_dict(self):
        data = {
            "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions",
            "users_url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/users",
            "teams_url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/teams",
            "apps_url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/apps",
            "users": [
                {
                    "login": "greenbonebot",
                    "id": 123,
                    "node_id": "MDQ6VXNlcjg1MjU0NjY2",
                    "avatar_url": "https://avatars.githubusercontent.com/u/85254666?v=4",
                    "gravatar_id": "",
                    "url": "https://api.github.com/users/greenbonebot",
                    "html_url": "https://github.com/greenbonebot",
                    "followers_url": "https://api.github.com/users/greenbonebot/followers",
                    "following_url": "https://api.github.com/users/greenbonebot/following{/other_user}",
                    "gists_url": "https://api.github.com/users/greenbonebot/gists{/gist_id}",
                    "starred_url": "https://api.github.com/users/greenbonebot/starred{/owner}{/repo}",
                    "subscriptions_url": "https://api.github.com/users/greenbonebot/subscriptions",
                    "organizations_url": "https://api.github.com/users/greenbonebot/orgs",
                    "repos_url": "https://api.github.com/users/greenbonebot/repos",
                    "events_url": "https://api.github.com/users/greenbonebot/events{/privacy}",
                    "received_events_url": "https://api.github.com/users/greenbonebot/received_events",
                    "type": "User",
                    "site_admin": False,
                }
            ],
            "teams": [],
            "apps": [],
        }

        restrictions = Restrictions.from_dict(data)
        self.assertEqual(
            restrictions.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions",
        )
        self.assertEqual(
            restrictions.users_url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/users",
        )
        self.assertEqual(
            restrictions.teams_url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/teams",
        )
        self.assertEqual(
            restrictions.apps_url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/apps",
        )
        user = restrictions.users[0]
        self.assertEqual(len(restrictions.users), 1)
        self.assertEqual(user.login, "greenbonebot")
        self.assertEqual(user.node_id, "MDQ6VXNlcjg1MjU0NjY2")
        self.assertEqual(
            user.avatar_url,
            "https://avatars.githubusercontent.com/u/85254666?v=4",
        )
        self.assertEqual(user.gravatar_id, "")
        self.assertEqual(user.url, "https://api.github.com/users/greenbonebot")
        self.assertEqual(user.html_url, "https://github.com/greenbonebot")
        self.assertEqual(
            user.followers_url,
            "https://api.github.com/users/greenbonebot/followers",
        )
        self.assertEqual(
            user.following_url,
            "https://api.github.com/users/greenbonebot/following{/other_user}",
        )
        self.assertEqual(
            user.gists_url,
            "https://api.github.com/users/greenbonebot/gists{/gist_id}",
        )
        self.assertEqual(
            user.starred_url,
            "https://api.github.com/users/greenbonebot/starred{/owner}{/repo}",
        )
        self.assertEqual(
            user.subscriptions_url,
            "https://api.github.com/users/greenbonebot/subscriptions",
        )
        self.assertEqual(
            user.organizations_url,
            "https://api.github.com/users/greenbonebot/orgs",
        )
        self.assertEqual(
            user.repos_url, "https://api.github.com/users/greenbonebot/repos"
        )
        self.assertEqual(
            user.events_url,
            "https://api.github.com/users/greenbonebot/events{/privacy}",
        )
        self.assertEqual(
            user.received_events_url,
            "https://api.github.com/users/greenbonebot/received_events",
        )
        self.assertEqual(user.type, "User")
        self.assertEqual(user.site_admin, False)
        self.assertEqual(restrictions.teams, [])
        self.assertEqual(restrictions.apps, [])


class RequiredStatusChecksTestCase(unittest.TestCase):
    def test_from_dict(self):
        data = {
            "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_status_checks",
            "strict": True,
            "checks": [
                {"context": "unittests", "app_id": 123},
                {"context": "linting"},
            ],
        }

        checks = RequiredStatusChecks.from_dict(data)

        self.assertEqual(
            checks.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_status_checks",
        )
        self.assertTrue(checks.strict)
        self.assertEqual(len(checks.checks), 2)
        self.assertEqual(checks.checks[0].context, "unittests")
        self.assertEqual(checks.checks[0].app_id, 123)
        self.assertEqual(checks.checks[1].context, "linting")
        self.assertIsNone(checks.checks[1].app_id)


class RequiredPullRequestReviewsTestCase(unittest.TestCase):
    def test_from_dict(self):
        data = {
            "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_pull_request_reviews",
            "dismiss_stale_reviews": True,
            "require_code_owner_reviews": True,
            "require_last_push_approval": True,
            "required_approving_review_count": 1,
            "dismissal_restrictions": {
                "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions",
                "users_url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions/users",
                "teams_url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions/teams",
                "users": [
                    {
                        "login": "greenbonebot",
                        "id": 123,
                        "node_id": "MDQ6VXNlcjg1MjU0NjY2",
                        "avatar_url": "https://avatars.githubusercontent.com/u/85254666?v=4",
                        "gravatar_id": "",
                        "url": "https://api.github.com/users/greenbonebot",
                        "html_url": "https://github.com/greenbonebot",
                        "followers_url": "https://api.github.com/users/greenbonebot/followers",
                        "following_url": "https://api.github.com/users/greenbonebot/following{/other_user}",
                        "gists_url": "https://api.github.com/users/greenbonebot/gists{/gist_id}",
                        "starred_url": "https://api.github.com/users/greenbonebot/starred{/owner}{/repo}",
                        "subscriptions_url": "https://api.github.com/users/greenbonebot/subscriptions",
                        "organizations_url": "https://api.github.com/users/greenbonebot/orgs",
                        "repos_url": "https://api.github.com/users/greenbonebot/repos",
                        "events_url": "https://api.github.com/users/greenbonebot/events{/privacy}",
                        "received_events_url": "https://api.github.com/users/greenbonebot/received_events",
                        "type": "User",
                        "site_admin": False,
                    }
                ],
                "teams": [
                    {
                        "name": "devops",
                        "id": 123,
                        "node_id": "T_kwDOAegUqc4AUtL1",
                        "slug": "devops",
                        "description": "Team responsible for DevOps",
                        "privacy": "closed",
                        "url": "https://api.github.com/organizations/321/team/123",
                        "html_url": "https://github.com/orgs/foo/teams/devops",
                        "members_url": "https://api.github.com/organizations/321/team/123/members{/member}",
                        "repositories_url": "https://api.github.com/organizations/321/team/123/repos",
                        "permission": "pull",
                        "parent": None,
                    }
                ],
                "apps": [],
            },
        }

        reviews = RequiredPullRequestReviews.from_dict(data)

        self.assertEqual(
            reviews.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_pull_request_reviews",
        )
        self.assertTrue(reviews.dismiss_stale_reviews)
        self.assertTrue(reviews.require_code_owner_reviews)
        self.assertTrue(reviews.require_last_push_approval)
        self.assertEqual(
            reviews.required_approving_review_count,
            1,
        )
        dismissal_restrictions = reviews.dismissal_restrictions
        self.assertEqual(
            dismissal_restrictions.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions",
        )
        self.assertEqual(
            dismissal_restrictions.users_url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions/users",
        )
        self.assertEqual(
            dismissal_restrictions.teams_url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions/teams",
        )
        self.assertEqual(dismissal_restrictions.apps, [])
        user = dismissal_restrictions.users[0]
        self.assertEqual(len(dismissal_restrictions.users), 1)
        self.assertEqual(user.login, "greenbonebot")
        self.assertEqual(user.node_id, "MDQ6VXNlcjg1MjU0NjY2")
        self.assertEqual(
            user.avatar_url,
            "https://avatars.githubusercontent.com/u/85254666?v=4",
        )
        self.assertEqual(user.gravatar_id, "")
        self.assertEqual(user.url, "https://api.github.com/users/greenbonebot")
        self.assertEqual(user.html_url, "https://github.com/greenbonebot")
        self.assertEqual(
            user.followers_url,
            "https://api.github.com/users/greenbonebot/followers",
        )
        self.assertEqual(
            user.following_url,
            "https://api.github.com/users/greenbonebot/following{/other_user}",
        )
        self.assertEqual(
            user.gists_url,
            "https://api.github.com/users/greenbonebot/gists{/gist_id}",
        )
        self.assertEqual(
            user.starred_url,
            "https://api.github.com/users/greenbonebot/starred{/owner}{/repo}",
        )
        self.assertEqual(
            user.subscriptions_url,
            "https://api.github.com/users/greenbonebot/subscriptions",
        )
        self.assertEqual(
            user.organizations_url,
            "https://api.github.com/users/greenbonebot/orgs",
        )
        self.assertEqual(
            user.repos_url, "https://api.github.com/users/greenbonebot/repos"
        )
        self.assertEqual(
            user.events_url,
            "https://api.github.com/users/greenbonebot/events{/privacy}",
        )
        self.assertEqual(
            user.received_events_url,
            "https://api.github.com/users/greenbonebot/received_events",
        )
        self.assertEqual(user.type, "User")
        self.assertEqual(user.site_admin, False)
        self.assertEqual(len(dismissal_restrictions.teams), 1)
        team = dismissal_restrictions.teams[0]
        self.assertEqual(team.name, "devops")
        self.assertEqual(team.id, 123)
        self.assertEqual(team.node_id, "T_kwDOAegUqc4AUtL1")
        self.assertEqual(team.slug, "devops")
        self.assertEqual(team.description, "Team responsible for DevOps")
        self.assertEqual(team.permission, Permission.PULL)
        self.assertEqual(team.privacy, TeamPrivacy.CLOSED)
        self.assertEqual(
            team.url, "https://api.github.com/organizations/321/team/123"
        )
        self.assertEqual(
            team.html_url, "https://github.com/orgs/foo/teams/devops"
        )
        self.assertEqual(
            team.members_url,
            "https://api.github.com/organizations/321/team/123/members{/member}",
        )
        self.assertEqual(
            team.repositories_url,
            "https://api.github.com/organizations/321/team/123/repos",
        )
        self.assertIsNone(team.parent)


class BranchProtectionTestCase(unittest.TestCase):
    def test_from_dict_minimal(self):
        data = {
            "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection",
            "required_signatures": {
                "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_signatures",
                "enabled": False,
            },
            "enforce_admins": {
                "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/enforce_admins",
                "enabled": False,
            },
            "required_linear_history": {"enabled": False},
            "allow_force_pushes": {"enabled": False},
            "allow_deletions": {"enabled": False},
            "block_creations": {"enabled": False},
            "required_conversation_resolution": {"enabled": False},
            "lock_branch": {"enabled": True},
            "allow_fork_syncing": {"enabled": False},
        }

        protection = BranchProtection.from_dict(data)

        self.assertEqual(
            protection.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection",
        )
        self.assertEqual(
            protection.required_signatures.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_signatures",
        )
        self.assertFalse(protection.required_signatures.enabled)
        self.assertEqual(
            protection.enforce_admins.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/enforce_admins",
        )
        self.assertFalse(protection.enforce_admins.enabled)
        self.assertFalse(protection.required_linear_history.enabled)
        self.assertFalse(protection.allow_force_pushes.enabled)
        self.assertFalse(protection.allow_deletions.enabled)
        self.assertFalse(protection.block_creations.enabled)
        self.assertFalse(protection.required_conversation_resolution.enabled)
        self.assertTrue(protection.lock_branch.enabled)
        self.assertFalse(protection.allow_fork_syncing.enabled)

        self.assertIsNone(protection.required_status_checks)
        self.assertIsNone(protection.required_pull_request_reviews)
        self.assertIsNone(protection.restrictions)

    def test_from_dict(self):
        data = {
            "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection",
            "required_status_checks": {
                "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_status_checks",
                "strict": True,
                "checks": [],
            },
            "restrictions": {
                "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions",
                "users_url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/users",
                "teams_url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/teams",
                "apps_url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/apps",
                "users": [
                    {
                        "login": "greenbonebot",
                        "id": 123,
                        "node_id": "MDQ6VXNlcjg1MjU0NjY2",
                        "avatar_url": "https://avatars.githubusercontent.com/u/85254666?v=4",
                        "gravatar_id": "",
                        "url": "https://api.github.com/users/greenbonebot",
                        "html_url": "https://github.com/greenbonebot",
                        "followers_url": "https://api.github.com/users/greenbonebot/followers",
                        "following_url": "https://api.github.com/users/greenbonebot/following{/other_user}",
                        "gists_url": "https://api.github.com/users/greenbonebot/gists{/gist_id}",
                        "starred_url": "https://api.github.com/users/greenbonebot/starred{/owner}{/repo}",
                        "subscriptions_url": "https://api.github.com/users/greenbonebot/subscriptions",
                        "organizations_url": "https://api.github.com/users/greenbonebot/orgs",
                        "repos_url": "https://api.github.com/users/greenbonebot/repos",
                        "events_url": "https://api.github.com/users/greenbonebot/events{/privacy}",
                        "received_events_url": "https://api.github.com/users/greenbonebot/received_events",
                        "type": "User",
                        "site_admin": False,
                    }
                ],
                "teams": [],
                "apps": [],
            },
            "required_pull_request_reviews": {
                "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_pull_request_reviews",
                "dismiss_stale_reviews": True,
                "require_code_owner_reviews": True,
                "require_last_push_approval": True,
                "required_approving_review_count": 1,
                "dismissal_restrictions": {
                    "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions",
                    "users_url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions/users",
                    "teams_url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions/teams",
                    "users": [
                        {
                            "login": "greenbonebot",
                            "id": 123,
                            "node_id": "MDQ6VXNlcjg1MjU0NjY2",
                            "avatar_url": "https://avatars.githubusercontent.com/u/85254666?v=4",
                            "gravatar_id": "",
                            "url": "https://api.github.com/users/greenbonebot",
                            "html_url": "https://github.com/greenbonebot",
                            "followers_url": "https://api.github.com/users/greenbonebot/followers",
                            "following_url": "https://api.github.com/users/greenbonebot/following{/other_user}",
                            "gists_url": "https://api.github.com/users/greenbonebot/gists{/gist_id}",
                            "starred_url": "https://api.github.com/users/greenbonebot/starred{/owner}{/repo}",
                            "subscriptions_url": "https://api.github.com/users/greenbonebot/subscriptions",
                            "organizations_url": "https://api.github.com/users/greenbonebot/orgs",
                            "repos_url": "https://api.github.com/users/greenbonebot/repos",
                            "events_url": "https://api.github.com/users/greenbonebot/events{/privacy}",
                            "received_events_url": "https://api.github.com/users/greenbonebot/received_events",
                            "type": "User",
                            "site_admin": False,
                        }
                    ],
                    "teams": [
                        {
                            "name": "devops",
                            "id": 123,
                            "node_id": "T_kwDOAegUqc4AUtL1",
                            "slug": "devops",
                            "description": "Team responsible for DevOps",
                            "privacy": "closed",
                            "url": "https://api.github.com/organizations/321/team/123",
                            "html_url": "https://github.com/orgs/foo/teams/devops",
                            "members_url": "https://api.github.com/organizations/321/team/123/members{/member}",
                            "repositories_url": "https://api.github.com/organizations/321/team/123/repos",
                            "permission": "pull",
                            "parent": None,
                        }
                    ],
                    "apps": [],
                },
            },
            "required_signatures": {
                "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_signatures",
                "enabled": True,
            },
            "enforce_admins": {
                "url": "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/enforce_admins",
                "enabled": True,
            },
            "required_linear_history": {"enabled": True},
            "allow_force_pushes": {"enabled": False},
            "allow_deletions": {"enabled": False},
            "block_creations": {"enabled": True},
            "required_conversation_resolution": {"enabled": True},
            "lock_branch": {"enabled": True},
            "allow_fork_syncing": {"enabled": False},
        }

        protection = BranchProtection.from_dict(data)

        self.assertEqual(
            protection.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection",
        )
        self.assertEqual(
            protection.required_signatures.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_signatures",
        )
        self.assertTrue(protection.required_signatures.enabled)
        self.assertEqual(
            protection.enforce_admins.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/enforce_admins",
        )
        self.assertTrue(protection.enforce_admins.enabled)
        self.assertTrue(protection.required_linear_history.enabled)
        self.assertFalse(protection.allow_force_pushes.enabled)
        self.assertFalse(protection.allow_deletions.enabled)
        self.assertTrue(protection.block_creations.enabled)
        self.assertTrue(protection.required_conversation_resolution.enabled)
        self.assertTrue(protection.lock_branch.enabled)
        self.assertFalse(protection.allow_fork_syncing.enabled)

        self.assertEqual(
            protection.required_status_checks.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_status_checks",
        )
        self.assertTrue(protection.required_status_checks.strict)
        self.assertEqual(protection.required_status_checks.checks, [])

        self.assertEqual(
            protection.required_pull_request_reviews.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/required_pull_request_reviews",
        )
        self.assertTrue(
            protection.required_pull_request_reviews.dismiss_stale_reviews
        )
        self.assertTrue(
            protection.required_pull_request_reviews.require_code_owner_reviews
        )
        self.assertTrue(
            protection.required_pull_request_reviews.require_last_push_approval
        )
        self.assertEqual(
            protection.required_pull_request_reviews.required_approving_review_count,
            1,
        )
        dismissal_restrictions = (
            protection.required_pull_request_reviews.dismissal_restrictions
        )
        self.assertEqual(
            dismissal_restrictions.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions",
        )
        self.assertEqual(
            dismissal_restrictions.users_url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions/users",
        )
        self.assertEqual(
            dismissal_restrictions.teams_url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/dismissal_restrictions/teams",
        )
        self.assertEqual(dismissal_restrictions.apps, [])
        user = dismissal_restrictions.users[0]
        self.assertEqual(len(dismissal_restrictions.users), 1)
        self.assertEqual(user.login, "greenbonebot")
        self.assertEqual(user.node_id, "MDQ6VXNlcjg1MjU0NjY2")
        self.assertEqual(
            user.avatar_url,
            "https://avatars.githubusercontent.com/u/85254666?v=4",
        )
        self.assertEqual(user.gravatar_id, "")
        self.assertEqual(user.url, "https://api.github.com/users/greenbonebot")
        self.assertEqual(user.html_url, "https://github.com/greenbonebot")
        self.assertEqual(
            user.followers_url,
            "https://api.github.com/users/greenbonebot/followers",
        )
        self.assertEqual(
            user.following_url,
            "https://api.github.com/users/greenbonebot/following{/other_user}",
        )
        self.assertEqual(
            user.gists_url,
            "https://api.github.com/users/greenbonebot/gists{/gist_id}",
        )
        self.assertEqual(
            user.starred_url,
            "https://api.github.com/users/greenbonebot/starred{/owner}{/repo}",
        )
        self.assertEqual(
            user.subscriptions_url,
            "https://api.github.com/users/greenbonebot/subscriptions",
        )
        self.assertEqual(
            user.organizations_url,
            "https://api.github.com/users/greenbonebot/orgs",
        )
        self.assertEqual(
            user.repos_url, "https://api.github.com/users/greenbonebot/repos"
        )
        self.assertEqual(
            user.events_url,
            "https://api.github.com/users/greenbonebot/events{/privacy}",
        )
        self.assertEqual(
            user.received_events_url,
            "https://api.github.com/users/greenbonebot/received_events",
        )
        self.assertEqual(user.type, "User")
        self.assertEqual(user.site_admin, False)
        self.assertEqual(len(dismissal_restrictions.teams), 1)
        team = dismissal_restrictions.teams[0]
        self.assertEqual(team.name, "devops")
        self.assertEqual(team.id, 123)
        self.assertEqual(team.node_id, "T_kwDOAegUqc4AUtL1")
        self.assertEqual(team.slug, "devops")
        self.assertEqual(team.description, "Team responsible for DevOps")
        self.assertEqual(team.permission, Permission.PULL)
        self.assertEqual(team.privacy, TeamPrivacy.CLOSED)
        self.assertEqual(
            team.url, "https://api.github.com/organizations/321/team/123"
        )
        self.assertEqual(
            team.html_url, "https://github.com/orgs/foo/teams/devops"
        )
        self.assertEqual(
            team.members_url,
            "https://api.github.com/organizations/321/team/123/members{/member}",
        )
        self.assertEqual(
            team.repositories_url,
            "https://api.github.com/organizations/321/team/123/repos",
        )
        self.assertIsNone(team.parent)

        self.assertEqual(
            protection.restrictions.url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions",
        )
        self.assertEqual(
            protection.restrictions.users_url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/users",
        )
        self.assertEqual(
            protection.restrictions.teams_url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/teams",
        )
        self.assertEqual(
            protection.restrictions.apps_url,
            "https://api.github.com/repos/foo/bar/branches/branch_protection/protection/restrictions/apps",
        )
        user = protection.restrictions.users[0]
        self.assertEqual(len(protection.restrictions.users), 1)
        self.assertEqual(user.login, "greenbonebot")
        self.assertEqual(user.node_id, "MDQ6VXNlcjg1MjU0NjY2")
        self.assertEqual(
            user.avatar_url,
            "https://avatars.githubusercontent.com/u/85254666?v=4",
        )
        self.assertEqual(user.gravatar_id, "")
        self.assertEqual(user.url, "https://api.github.com/users/greenbonebot")
        self.assertEqual(user.html_url, "https://github.com/greenbonebot")
        self.assertEqual(
            user.followers_url,
            "https://api.github.com/users/greenbonebot/followers",
        )
        self.assertEqual(
            user.following_url,
            "https://api.github.com/users/greenbonebot/following{/other_user}",
        )
        self.assertEqual(
            user.gists_url,
            "https://api.github.com/users/greenbonebot/gists{/gist_id}",
        )
        self.assertEqual(
            user.starred_url,
            "https://api.github.com/users/greenbonebot/starred{/owner}{/repo}",
        )
        self.assertEqual(
            user.subscriptions_url,
            "https://api.github.com/users/greenbonebot/subscriptions",
        )
        self.assertEqual(
            user.organizations_url,
            "https://api.github.com/users/greenbonebot/orgs",
        )
        self.assertEqual(
            user.repos_url, "https://api.github.com/users/greenbonebot/repos"
        )
        self.assertEqual(
            user.events_url,
            "https://api.github.com/users/greenbonebot/events{/privacy}",
        )
        self.assertEqual(
            user.received_events_url,
            "https://api.github.com/users/greenbonebot/received_events",
        )
        self.assertEqual(user.type, "User")
        self.assertEqual(user.site_admin, False)
        self.assertEqual(protection.restrictions.teams, [])
        self.assertEqual(protection.restrictions.apps, [])
